#ifndef	__RULE_LINE_API_H__
#define	__RULE_LINE_API_H__


#define	MAXPEAK			5
#define	MAXLINEGAP		100

struct LINE{
	POINT	StPnt;		// Start point
	POINT	EdPnt;		// End point
	int		nType;		// Line type. 0 for virtual lines
	int		nPixel;		// Number of black pixels on the line
};

struct HMM_MODEL{
	double 	TextPeakProb[MAXPEAK];		//Probability for text to generate peaks on the skewed horizotnal projection
	double	LinePeakProb[MAXPEAK];		//Probability for background lines to generate peaks on the skewed horizotnal projection
	double  LineGapDist[MAXLINEGAP];	//Vertical line gap distribution
	int		nMinLineGap, nMaxLineGap;
};


class __declspec(dllexport) CRLLineDetection
{
private:
	HMM_MODEL	m_HMM_Model;		//HMM model
	LINE		*m_pLine;			//Rule lines
	int			m_nLine;			//Number of rule lines
	bool		m_bLine;			//Whether there are rule lines on the document

public:
	CRLLineDetection();
	~CRLLineDetection();

public:
	int RLSetImage( char *fnImg );			//Set image file name for rule line detection
	int RLSetImage( BYTE *pImgData, int w, int h );	//Set image from memory for rule line detection
	int RLSetHMMModel( char *fnHMM_Model );	//Set HMM model
	int RLLineDetect( );					//Rule line detection
	bool RLLineExist( );					//Whether rule lines exist
	int RLGetLines( LINE *&pLine, int &nLine );	//Get detected lines
	int RLLineRemove(  );	//Rule line removal
	int RLGetCleanImage( BYTE *&pCleanImg, int &w, int &h); //Get cleaned image after rule line removal
	int RLSaveCleanImage( char *fnCleanImg ); //Save cleaned image
};

class __declspec(dllexport) CRLTrainHMM
{
private:
	char		m_fnTrainHMMConfig[MAX_PATH];	//Configure file of the training set
	HMM_MODEL	m_HMM_Model;		//Trained HMM model

public:
	CRLTrainHMM();
	~CRLTrainHMM();

public:
	int RLSetTrainConfig( char *fnTrainConfig );//Set configure file
	int RLTrainRuleLineModel( );				//Train HMM model
	int RLOutputRuleLineModel( char *fnHMM_Model );	//Output HMM model
};

#endif	//__RULE_LINE_API_H__