/////////////////////////////////////////////////////////////////////////
// File NAME:		TextIDAPI.h
// File Function:	Head file for the API for text identification
//
//				Developed by:Yefeng Zheng
//					All Right Reserved
/////////////////////////////////////////////////////////////////////////

#ifndef	__TEXT_ID_API_H__
#define __TEXT_ID_API_H__

#define	TYPE_TOTAL_CLASS    3

#define TYPE_PRINT_TEXT		1
#define TYPE_HANDWRITING	2
#define	TYPE_NOISE			3
#define	TYPE_OTHERS			4

#define MRF_SIZE_LEVEL		2

#ifndef	WORD_ZONE_TYPE
#define	WORD_ZONE_TYPE
typedef	struct {
//Bounding box of the word zone
  	int x;		
  	int y;
  	int w;
  	int h;
//Classification result
	int nType;
//Reserved
	int nIndex;
//Classification confidence for the first choice
	double	nConf;
//Classification confidence for all choices
	double	nClassConf[TYPE_TOTAL_CLASS];
}WORD_ZONE;
#endif

//Fisher model
typedef struct {
	int	nDimension;		//Feature dimension
	int	nClassNum;		//Number of classes
	int	*nClassLabel;	//All class labels
	double	**Eff;		//Fisher classifier projection coefficients
	double	*nMean1;	//Projection mean for class 1
	double	*nVar1;		//Projection variance for class 1
	double	*nMean2;	//Projection mean for class 2
	double	*nVar2;		//Projection variance for class 2
	int		*nClassLabel1;	//Label for class 1
	int		*nClassLabel2;	//Label for class 2
	double	*nTH;		//Threshold
}CLASSIFIER;


//MRF model
typedef struct {
	int		nZone[MRF_SIZE_LEVEL];		//Number of zones for each size level
	int		nLabel[MRF_SIZE_LEVEL][TYPE_TOTAL_CLASS];	//Number of zones for each class and each size level
	//Clique frequency for Cp
	int		****nPatternP;
	//Clique frequency for Cn
	int		******nPatternN;
	int		nCliqueType;					//Clique type
	int		nClassLabel[TYPE_TOTAL_CLASS];	//Class labels
	int		nClassNum;						//Number of classes
} MRF_MODEL;

///////////////////////////////////////////////////////////////////////////////////////
class __declspec(dllexport) CTILayerSeparate {
private:
	WORD_ZONE	*m_pWord;		//Segmented words
	int			m_nWord;		//Number of words

	CLASSIFIER	m_Classifier;	//Classifier
	MRF_MODEL	m_MRF_Model;	//MRF model for post-processing

public:
	CTILayerSeparate(void);
	~CTILayerSeparate(void);

public:
	//Set image from file for processing
	int TISetImage(char *fnImg);
	//Set image from memory for processing
	int TISetImage(BYTE *pImgData, int w, int h);
	//Set MRF model for post-processing
	int TISetMRFModel(char *fnMRFModel);
	//Set classifier
	int TISetClassifier( char *fnClassifier );

	//Word segmentation
	int TIWordSegmentation( );
	//performing word classification
	int TIClassification( );
	//MRF-based post processing
	int TIMRFPostProcessing( );

	//This function provides a way for users can use his/her own module to perform segmentation, classification of logo, figure, etc., and then fill information to the structure of WORD.
	int TISetWord(WORD_ZONE *pWord, int nWord);
	//Similar to the previous function, but the custom's result is stored in ZONE file.
	int TISetWord(char *fnCustom);
	//Get the segmentation and classification results
	int TIGetWord(WORD_ZONE *&pWord, int &nWord );

	//Output result to ZONE file
	int TIOutputZone( char *fnZone );
	//Get layer
	int TIGetLayer( BYTE *&pImgData, int &w, int &h, int nType );
	//Save layer
	int TISaveLayer( char *fnLayerImg, int nType );

private:
//	int TITestCorrectness( );	// This is an internal test function.
								// Test if all pixels are assigned to one and only one layer.
};

////////////////////////////////////////////////////////////////////////////////////////
class __declspec(dllexport) CTITrainClassifier
{
private:
	char		m_fnTrainClassifierConfig[MAX_PATH];	//Configure file of the training set
	CLASSIFIER	m_Classifier;						//Trained classifier

public:
	CTITrainClassifier(void);
	~CTITrainClassifier(void);

public:
	int TISetTrainConfig( char *fnTrainConfig );	//Set the configure file of the training set
	int TITrainClassifier( );						//Training the classifier
	int TIOutputClassifier( char *fnClassifier );	//Output trained classifier
};

/////////////////////////////////////////////////////////////////////////////////////////
class __declspec(dllexport) CTITrainMRFModel
{
private:
	char 		m_fnTrainMRFConfig[MAX_PATH];	//Configure file of the training set
	MRF_MODEL	m_MRF_Model;				//Trained MRF model

public:
	CTITrainMRFModel(void);
	~CTITrainMRFModel(void);

public:
	int TISetTrainConfig( char *fnTrainConfig );	//Set configure file
	int TITrainMRFModel( );							//Training the MRF model
	int TIOutputMRFModel( char *fnModel );			//Output trained MRF model
};

#endif //__TEXT_ID_API_H__