/////////////////////////////////////////////////////////////////////////
// File NAME:		EvalTree.cpp
// File Function:	C4.5 classification tree. Modified from the original 
//					source	code of C4.5.
//
//				Developed by: Yefeng Zheng
//			University of Maryland, College Park
//					All Right Reserved
/////////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Tree.h"

short		MaxAtt, MaxClass, MaxDiscrVal;
char		Delimiter;
DiscrValue	*MaxAttVal=NULL;
char		*SpecialStatus=NULL;
String		*ClassName=NULL,	*AttName=NULL, **AttValName=NULL;
float		ClassSum[100];		/* ClassSum[c] = total weight of class c */


void  StreamIn(FILE *TRf, String s, int n)
{
    while ( n-- ) *s++ = getc(TRf);
}

/*************************************************************************/
/*									 */
/*	Retrieve tree from saved characters				 */
/*									 */
/*************************************************************************/


Tree InTree( FILE *TRf)
/*   -------  */
{
    Tree T;
    DiscrValue v;
    int Bytes;

    T = (Tree) malloc(sizeof(TreeRec));

    StreamIn(TRf, (char *) &T->NodeType, sizeof(short));
    StreamIn(TRf, (char *) &T->Leaf, sizeof(ClassNo));
    StreamIn(TRf, (char *) &T->Items, sizeof(ItemCount));
    StreamIn(TRf, (char *) &T->Errors, sizeof(ItemCount));

    T->ClassDist = (ItemCount *) calloc(MaxClass+1, sizeof(ItemCount));
    StreamIn(TRf, (char *) T->ClassDist, (MaxClass + 1) * sizeof(ItemCount));

    if ( T->NodeType )
    {
		StreamIn(TRf, (char *) &T->Tested, sizeof(Attribute));
		StreamIn(TRf, (char *) &T->Forks, sizeof(short));

		switch ( T->NodeType )
		{
			case BrDiscr:
			break;

			case ThreshContin:
			StreamIn(TRf, (char *) &T->Cut, sizeof(float));
			StreamIn(TRf, (char *) &T->Lower, sizeof(float));
			StreamIn(TRf, (char *) &T->Upper, sizeof(float));
			break;

			case BrSubset:
			T->Subset = (Set *) calloc(T->Forks + 1, sizeof(Set));

			Bytes = (MaxAttVal[T->Tested]>>3) + 1;
			ForEach(v, 1, T->Forks)
			{
				T->Subset[v] = (Set) malloc(Bytes);
				StreamIn(TRf, (char *) T->Subset[v], Bytes);
			}
		}

		T->Branch = (Tree *) calloc(T->Forks + 1, sizeof(Tree));
		ForEach(v, 1, T->Forks)
		{
			T->Branch[v] = InTree(TRf);
		}
    }

    return T;
}

/*************************************************************************/
/*									 */
/*	Retrieve entire decision tree with extension Extension		 */
/*									 */
/*************************************************************************/


Tree GetTree(char *fnTree)
/*   --------  */
{
    Tree Hold;

	FILE *TRf;
	if ( ! ( TRf = fopen(fnTree, "rb") ) ) 
		return 0;
    if ( ! TRf || getc(TRf) == EOF ) return 0;

    Hold = InTree( TRf);

    return Hold;
}

/*************************************************************************/
/*									 */
/*	Free up space taken up by tree Node				 */
/*									 */
/*************************************************************************/
void  ReleaseTree(Tree Node)
{
    DiscrValue v;

    if ( Node->NodeType )
    {
		ForEach(v, 1, Node->Forks)
		{
			ReleaseTree(Node->Branch[v]);
		}

		free(Node->Branch);

		if ( Node->NodeType == BrSubset )
		{
			free(Node->Subset);
		}

    }

    free(Node->ClassDist);
    free(Node);
}


/*************************************************************************/
/*									 */
/*	Allocate space then copy string into it				 */
/*									 */
/*************************************************************************/

String CopyString(String x)
{
    char *s;

    s = (char *) calloc(strlen(x)+1, sizeof(char));
    strcpy(s, x);
    return s;
}

/*************************************************************************/
/*									 */
/*  Read a name from file f into string s, setting Delimiter.		 */
/*									 */
/*  - Embedded periods are permitted, but periods followed by space	 */
/*    characters act as delimiters.					 */
/*  - Embedded spaces are permitted, but multiple spaces are replaced	 */
/*    by a single space.						 */
/*  - Any character can be escaped by '\'.				 */
/*  - The remainder of a line following '|' is ignored.			 */
/*									 */
/*************************************************************************/


Boolean ReadName(FILE *f, String s)
{
    register char *Sp=s;
    register int c;

    /*  Skip to first non-space character  */

    while ( ( c = getc(f) ) == '|' || Space(c) )
    {
	if ( c == '|' ) SkipComment;
    }

    /*  Return false if no names to read  */

    if ( c == EOF )
    {
	Delimiter = EOF;
	return 0;
    }

    /*  Read in characters up to the next delimiter  */

    while ( c != ':' && c != ',' && c != '\n' && c != '|' && c != EOF )
    {
	if ( c == '.' )
	{
	    if ( ( c = getc(f) ) == '|' || Space(c) ) break;
	    *Sp++ = '.';
	}

	if ( c == '\\' )
	{
	    c = getc(f);
	}

	*Sp++ = c;

	if ( c == ' ' )
	{
	    while ( ( c = getc(f) ) == ' ' )
		;
	}
	else
	{
	    c = getc(f);
	}
    }

    if ( c == '|' ) SkipComment;
    Delimiter = c;

    /*  Strip trailing spaces  */

    while ( Space(*(Sp-1)) ) Sp--;

    *Sp++ = '\0';
    return 1;
}


/*************************************************************************/
/*									 */
/*  Read the names of classes, attributes and legal attribute values.	 */
/*  On completion, these names are stored in:				 */
/*	ClassName	-  class names					 */
/*	AttName		-  attribute names				 */
/*	AttValName	-  attribute value names			 */
/*  with:								 */
/*	MaxAttVal	-  number of values for each attribute		 */
/*									 */
/*  Other global variables set are:					 */
/*	MaxAtt		-  maximum attribute number			 */
/*	MaxClass	-  maximum class number				 */
/*	MaxDiscrVal	-  maximum discrete values for any attribute	 */
/*									 */
/*  Note:  until the number of attributes is known, the name		 */
/*	   information is assembled in local arrays			 */
/*									 */
/*************************************************************************/
int GetNames( char *Fn)
/*  ---------  */
{
    FILE *Nf;
    char Buffer[1000];
    DiscrValue v;
    int AttCeiling=100, ClassCeiling=100, ValCeiling;

    /*  Open names file  */
    if ( ! ( Nf = fopen(Fn, "rt") ) ) return -1;

    /*  Get class names from names file  */

    ClassName = (String *) calloc(ClassCeiling, sizeof(String));
    MaxClass = -1;
    do
    {
		ReadName(Nf, Buffer);

		if ( ++MaxClass >= ClassCeiling)
		{
			ClassCeiling += 100;
			ClassName = (String *) realloc(ClassName, ClassCeiling*sizeof(String));
		}
		ClassName[MaxClass] = CopyString(Buffer);
    }
    while ( Delimiter == ',' );

    /*  Get attribute and attribute value names from names file  */

    AttName = (String *) calloc(AttCeiling, sizeof(String));
    MaxAttVal = (DiscrValue *) calloc(AttCeiling, sizeof(DiscrValue));
    AttValName = (String **) calloc(AttCeiling, sizeof(String *));
    SpecialStatus = (char *) malloc(AttCeiling);

    MaxAtt = -1;
    while ( ReadName(Nf, Buffer) )
    {
	if ( Delimiter != ':' ) return -1;

	if ( ++MaxAtt >= AttCeiling )
	{
	    AttCeiling += 100;
	    AttName = (String *) realloc(AttName, AttCeiling*sizeof(String));
	    MaxAttVal = (DiscrValue *) realloc(MaxAttVal, AttCeiling*sizeof(DiscrValue));
	    AttValName = (String **) realloc(AttValName, AttCeiling*sizeof(String *));
	    SpecialStatus = (char *) realloc(SpecialStatus, AttCeiling);
	}

	AttName[MaxAtt] = CopyString(Buffer);
	SpecialStatus[MaxAtt] = 0;
	MaxAttVal[MaxAtt] = 0;
	ValCeiling = 100;
	AttValName[MaxAtt] = (String *) calloc(ValCeiling, sizeof(String));

	do
	{
	    if ( ! ( ReadName(Nf, Buffer) ) ) return -1;

	    if ( ++MaxAttVal[MaxAtt] >= ValCeiling )
	    {
		ValCeiling += 100;
		AttValName[MaxAtt] =
		    (String *) realloc(AttValName[MaxAtt], ValCeiling*sizeof(String));
	    }

	    AttValName[MaxAtt][MaxAttVal[MaxAtt]] = CopyString(Buffer);
	}
	while ( Delimiter == ',' );

	if ( MaxAttVal[MaxAtt] == 1 )
	{
	    /*  Check for special treatment  */

	    if ( ! strcmp(Buffer, "continuous") )
	    {}
	    else
	    if ( ! memcmp(Buffer, "discrete", 8) )
	    {
		SpecialStatus[MaxAtt] = DISCRETE;

		/*  Read max values, reserve space and check MaxDiscrVal  */

		v = atoi(&Buffer[8]);
		if ( v < 2 )
		{
		    exit(1);
		}

		AttValName[MaxAtt] =
		    (String *) realloc(AttValName[MaxAtt], (v+2)*sizeof(String));
		AttValName[MaxAtt][0] = (char *) v;
		if ( v > MaxDiscrVal ) MaxDiscrVal = v;
	    }
	    else
	    if ( ! strcmp(Buffer, "ignore") )
	    {
		SpecialStatus[MaxAtt] = IGNORE_ATT;
	    }
	    else
	    {
		/*  Cannot have only one discrete value for an attribute  */

		return -1;
	    }

	    MaxAttVal[MaxAtt] = 0;
	}
	else
	if ( MaxAttVal[MaxAtt] > MaxDiscrVal ) MaxDiscrVal = MaxAttVal[MaxAtt];
    }

    fclose(Nf);
	return 0;
}

/*************************************************************************/
/*                                                              	 */
/*  Classify a case description using the given subtree by adjusting	 */
/*  the value ClassSum for each class					 */
/*                                                              	 */
/*************************************************************************/
void Classify(Description CaseDesc, Tree T, float Weight)
{
    DiscrValue v, dv;
    float Cv;
    Attribute a;
    ClassNo c;

    switch ( T->NodeType )
    {
        case 0:  /* leaf */

	    if ( T->Items > 0 )
	    {
		/*  Update from ALL classes  */

		ForEach(c, 0, MaxClass)
		{
		    if ( T->ClassDist[c] )
		    {
			ClassSum[c] += Weight * T->ClassDist[c] / T->Items;
		    }
		}
	    }
	    else
	    {
		ClassSum[T->Leaf] += Weight;
	    }

	    return;

	case BrDiscr:  /* test of discrete attribute */

	    a = T->Tested;
	    v = DVal(CaseDesc, a);

	    if ( v && v <= T->Forks )	/*  Make sure not new discrete value  */
	    {
		Classify(CaseDesc, T->Branch[v], Weight);
	    }
	    else
	    {
		ForEach(v, 1, T->Forks)
		{
		    Classify(CaseDesc, T->Branch[v], 
			     (Weight * T->Branch[v]->Items) / T->Items);
		}
	    }

	    return;

	case ThreshContin:  /* test of continuous attribute */

	    a = T->Tested;
	    Cv = CVal(CaseDesc, a);

	    if ( Cv == Unknown )
	    {
		ForEach(v, 1, 2)
		{
		    Classify(CaseDesc, T->Branch[v], 
			     (Weight * T->Branch[v]->Items) / T->Items);
		}
	    }
	    else
	    {
		v = ( Cv <= T->Cut ? 1 : 2 );
		Classify(CaseDesc, T->Branch[v], Weight);
	    }

	    return;

	case BrSubset:  /* subset test on discrete attribute  */

	    a = T->Tested;
	    dv = DVal(CaseDesc, a);

	    if ( dv )
	    {
		ForEach(v, 1, T->Forks)
		{
		    if ( In(dv, T->Subset[v]) )
		    {
			Classify(CaseDesc, T->Branch[v], Weight);

			return;
		    }
		}
	    }

	    /*  Value unknown or not found in any of the subsets  */

	    ForEach(v, 1, T->Forks)
	    {
		Classify(CaseDesc, T->Branch[v], 
		         (Weight * T->Branch[v]->Items) / T->Items);
	    }

	    return;
    } 
}

/*************************************************************************/
/*                                                              	 */
/*  Categorize a case description using the given decision tree		 */
/*                                                              	 */
/*************************************************************************/

ClassNo Category(Description CaseDesc, Tree DecisionTree) 
{ 
    ClassNo c, BestClass;

    ForEach(c, 0, MaxClass)
    {
		ClassSum[c] = 0;
    }

    Classify(CaseDesc, DecisionTree, 1.0);

    BestClass = 0;
    ForEach(c, 0, MaxClass)
    {
		if ( ClassSum[c] > ClassSum[BestClass] ) BestClass = c;
    }
    return BestClass;
}

/*************************************************************************/
/*									 */
/*	Free memory storing NAMES		 */
/*									 */
/*************************************************************************/
int FreeNames( )
{
	int i;
	if( SpecialStatus != NULL )
		free( SpecialStatus );
	if( ClassName != NULL)
	{
		for( i=0; i<=MaxClass; i++ )
			free( ClassName[i] );
		free( ClassName );
	}
	if( AttName != NULL )
	{
		for( i=0; i<=MaxAtt; i++ )
			free( AttName[i] );
		free( AttName );
	}
	if( AttValName != NULL )
	{
		for( i=0; i<=MaxAtt; i++ )
		{
			for( int j=0; j<100; j++ )
			{
				if( AttValName[i][j] != NULL )
					free( AttValName[i][j] );
			}
			free( AttValName[i] );
		}
		free( AttValName );
	}
	if( MaxAttVal != NULL )
		free( MaxAttVal );
	return 0;
}

/*************************************************************************/
/*																		*/
/*	Read the trained classification  tree from file						*/
/*																		*/
/*************************************************************************/
int ReadTree( char *fnTree, Tree &DecisionTree )
{
	char	fnNames[256];
    strcpy ( fnNames, fnTree ) ;
    for ( int i=strlen(fnNames)-1 ; i>=0 ; i-- )
    {
        if ( fnNames[i] == '.' )
        {
            fnNames[i] = '\0' ;
            break ;
        }
        else if ( fnNames[i] == '\\' )
            break ;
    }
    strcat ( fnNames, "." ) ;
    strcat ( fnNames, "names" ) ;
	
	if( GetNames( fnNames ) != 0 )
		return -1;
	DecisionTree = GetTree( fnTree );
	if( DecisionTree == NULL )
		return -1;
	FreeNames();
	return 0;
}

/*************************************************************************/
/*																		 */
/*	Free memory of the classification tree															 */
/*																		 */
/*************************************************************************/
int FreeTree( Tree DecisionTree )
{
	ReleaseTree( DecisionTree );
	return 0;
}

/*************************************************************************/
/*																		*/
/*	Perform classification for one sample								*/
/*																		*/
/*************************************************************************/
int ClassTree( double *Feature, int nFeature, Tree DecisionTree )
{
    AttValue CaseDesc[20]; 
	for( int i=0; i<nFeature; i++ )
		CaseDesc[i]._cont_val = (float)Feature[i];
	
	return Category(CaseDesc, DecisionTree);
}
