/////////////////////////////////////////////////////////////////////////
// File NAME:		ImageObj.h
// File Function:	Read and write image file
//
//				Developed by: Yefeng Zheng
//					All Right Reserved
/////////////////////////////////////////////////////////////////////////

#ifndef __IMAGEOBJ_H__
#define __IMAGEOBJ_H__

//Image type
enum { DIB_1BIT, DIB_4BIT, DIB_8BIT, DIB_16BIT, DIB_24BIT, DIB_32BIT, DIB_UNKNOWN };
//Palette type
enum{ WHITEISZERO, BLACKISZERO, RGB, PALETTE };

class CImage : public CObject
{

DECLARE_SERIAL(CImage)
public:
	CImage();
	~CImage();
	FreeMem();	//Free memory

protected:
    BOOL	m_bImported;//If the image data ara allocate by other class.
						//If yes, do not free the image memory 
	HGLOBAL m_hImg;		//Image handle
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
	BOOL	m_bUpSideDown;//TRUE -- Upside down DIB format
	BYTE	*m_pImg;		//Pointer to image data
	int		m_nWidth;		//Image width
	int		m_nHeight;		//Image height
	int		m_nWidthByte;	//Image width in bytes
	int		m_nBitCount;	//Bit count per pixel

	unsigned char lut[256];	//Test how many black pixels in one byte
public:
//////////////////////////////////////////////////////////////////////////////////////////////////
//	Initailize image
//////////////////////////////////////////////////////////////////////////////////////////////////
    //Initialize to void
	BOOL Initialize() ;
	//Initialize from other CImage instance
    BOOL Initialize ( CImage &Image) ;
	BOOL Initialize ( int nWidth, int nHeight, int nBitCount, int nPlanes );
	BOOL Initialize ( HGLOBAL hDIBRawImg, CRect rcRegion, int nImgType );
	BOOL Initialize ( HDC hdc, CBitmap& bmp ) ;
    BOOL Initialize ( BYTE *lpRawImg, int nWidth, int nLineSizeInBytes, int nHeight, int nImgType, BOOL bUpsideDown = FALSE ) ;
	//Initialize array lut[0-255], which calculate the number of black pixels in a byte
	void InitBlackPixelsInByte();
	//Clear image
	int	 Clear( );
/////////////////////////////////////////////////////////////////////////////////////////
//	Get image information
/////////////////////////////////////////////////////////////////////////////////////////
public:
	//Return lut
	BYTE *GetBlackPixelsInByteMask();
	//Get m_bUpsideDown
	int GetOrientation();
	//Get image type
    int GetType() ;
	//Get image width
    int GetWidth() ;
	//Get image height
	int GetHeight() ;
	//Get the number of bytes to represent an image line
    int GetLineSizeInBytes() ;
	//Return image data handle in the memory
	HGLOBAL GethImg() ;
	//Get black pixel ratio
    double GetBlackPixelRatio() ;
	//Retrieval the palette to see if the image is color
	BOOL IsColor();
protected:
	//Get the number of bytes to represent an image line
	int GetLineSizeInBytes(LPBITMAPINFO lpbi);
	//Get the palette size used
    int GetPaletteSize ( LPBITMAPINFO lpbi ) ;

///////////////////////////////////////////////////////////////////////////////////
//	Read and write image
//////////////////////////////////////////////////////////////////////////////////
public:
	//Lock image in the memory, return pointer to the image data
    BYTE *LockRawImg() ;
	//Release the lock
    int UnlockRawImg() ;
	//Read image from file
	int Read ( char* csFileName, BOOL bInvert=TRUE ) ;
	//Write image to file
    int Write ( CString csFileName ) ;
protected:
	//Return raw image data pointer from BITMAPINFO structure
	BYTE *GetRawImgPtr ( LPBITMAPINFO lpbi ) ;
	//Get file extension to judge image type
	CString GetFileExtension ( CString FileName );

/////////////////////////////////////////////////////////////////////////////////
//	Others
////////////////////////////////////////////////////////////////////////////////
public:
	//Test if need to invert image bits
	BOOL NeedInvertBits();
	//Invert image bits
    int InvertBits () ;
	//Test if need to invert image palette
	BOOL NeedInvertPallete() ;
	//Invert image palette
    int	InvertPallete() ;
	//Set image palette
    int SetPalette(LPBITMAPINFO lpbi) ;
	//Set image palette
    int SetPalette(RGBQUAD *pColors, int start, int end ) ;
	//Show full image to screen
    int ShowFullImgDIB ( HDC hdc, int xd, int yd ) ;
	//Show full image to screen, stretch the image to fill specific region
	int ShowStretchImgDIB ( HDC hdc, int xd, int yd, int width, int height);
};

#endif  // __IMAGEOBJ_H__
