#ifndef __TREE_H__
#define	__TREE_H__

typedef  int	ItemNo;		/* data item number */
typedef  float	ItemCount;	/* count of (partial) items */
typedef  char	Boolean, *String, *Set;

typedef  short	ClassNo,	/* class number, 0..MaxClass */
				DiscrValue;	/* discrete attribute value (0 = ?) */
typedef  short	Attribute;	/* attribute number, 0..MaxAtt */

typedef  union  _attribute_value
	 {
	    DiscrValue	_discr_val;
	    float	_cont_val;
	 }AttValue, *Description;

typedef  struct _tree_record *Tree;
typedef  struct _tree_record
	 {
	    short	NodeType;	/* 0=leaf 1=branch 2=cut 3=subset */
	    ClassNo	Leaf;		/* most frequent class at this node */
	    ItemCount	Items,		/* no of items at this node */
				*ClassDist,	/* class distribution of items */
	    		Errors;		/* no of errors at this node */
	    Attribute	Tested; 	/* attribute referenced in test */
	    short	Forks;		/* number of branches at this node */
	    float	Cut,		/* threshold for continuous attribute */
		  		Lower,		/* lower limit of soft threshold */
		  		Upper;		/* upper limit ditto */
	    Set     *Subset;	/* subsets of discrete values  */
	    Tree	*Branch;	/* Branch[x] = (sub)tree for outcome x */
	 }TreeRec;

#define  BrDiscr	1	/* node types:	branch */
#define  ThreshContin	2	/*		threshold cut */
#define  BrSubset	3	/*		subset test */

#define  IGNORE_ATT		1	/* special attribute status: do not use */
#define  DISCRETE		2	/* ditto: collect values as data read */

#define  Unknown  -999		/* unknown value for continuous attribute */

#define	 ForEach(v,f,l)		for(v=f ; v<=l ; ++v) 
#define  Space(s)	(s == ' ' || s == '\n' || s == '\t')
#define  SkipComment	while ( ( c = getc(f) ) != '\n' )

#define  CVal(Case,Attribute)   Case[Attribute]._cont_val
#define  DVal(Case,Attribute)   Case[Attribute]._discr_val
#define  Class(Case)		Case[MaxAtt+1]._discr_val

#define	 Bit(b)			(1 << (b))
#define	 In(b,s)		((s[(b) >> 3]) & Bit((b) & 07))


int ReadTree(char *fnTree, Tree &DecisionTree);
int FreeTree(Tree DecisiontTree);
int ClassTree( double *Feature, int nFeature, Tree DecisionTree);

#endif //__TREE_H__