////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		define.h
// File Function:	Definition of some common structures
//
//				Developed by: Yefeng Zheng
//			   First created: Jan, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _DEFINE_H__
#define	_DEFINE_H__

#include "ArabicTextIDAPI.h"

#ifndef M_PI_2
#define	M_PI_2		1.57079632679489661923
#endif

#define M_PI		3.14159265358979323846

#define SIZE_LEVEL			3
#define	SIZE_TINY			1	//Less than one character size, for punctation
#define SIZE_SMALL			2	//Less than two character size
#define SIZE_LARGE			3	//Larger than two character size

#define CLIQUE_PRINT		1
#define	CLIQUE_NOISE		2
#define	CLIQUE_ALL			3

#define	FEATURE_DIMENSION	140

typedef struct ZONE{
  	int x;
  	int y;
  	int w;
  	int h;
  	int flg;
	int nType;
  	int *child;
	int	child_count;
	int index;
	int nBlackPixels;

	int	nSize;
	double	nConf;
	double  nClassConf[TYPE_TOTAL_CLASS];
}ZONE;

//////////////////////////////////////////////////////////////////////////////////////////
// Structures defined for word segmentation
//////////////////////////////////////////////////////////////////////////////////////////
typedef struct node_docstrum
{
	int v;
  	struct node_docstrum *next;
} NODE_DOCSTRUM;

typedef struct dis {
  	double d;
  	double a;
  	int    i;
} DIS;

typedef struct dist{
	int		d1;
	int		d2;
} DIST;

typedef struct est_para {
  	double or;
  	int cs;
  	int ts;
	int cw;
	int ch;
} EST_PARA;

typedef struct docstrum_param{	/* Docstrum algorithm parameter data structure */
        int     knn;        	/* number of nearest connected components to be clustered */
        int     lsize;      	/* low size threshold for connected component */
        int     hsize;      	/* high size threshold for connected component */
        int     hatol;      	/* horizontal angle tolerance */
        int     vatol;      	/* vertical angle tolerance */
        float   tcc;        /* threshold for clustering connected component into lines */
        float   par;        /* parallel threshold for clustering lines into zones */
        float   per;        /* perpendicular threshold for clustering lines into zones */
        float   scr;        /* super/sub script distance threshold */
        float   dis;        /* connected component size grouping threshold */
} DOCSTRUM_PARAM;

//Flip candidate
typedef struct {
	double	nConf;			//Flip confidence
	int		nFlippedType;	//Label after flip
	int		nIndex;			//Block candidate for flip
} FLIP_CANDIDATE;

#endif //_DEFINE_H__