/*====================================================================*
 -  Copyright (C) 2001 Leptonica.  All rights reserved.
 -  This software is distributed in the hope that it will be
 -  useful, but with NO WARRANTY OF ANY KIND.
 -  No author or distributor accepts responsibility to anyone for the
 -  consequences of using this software, or for whether it serves any
 -  particular purpose or works at all, unless he or she says so in
 -  writing.  Everyone is granted permission to copy, modify and
 -  redistribute this source code, for commercial or non-commercial
 -  purposes, with the following restrictions: (1) the origin of this
 -  source code must not be misrepresented; (2) modified versions must
 -  be plainly marked as such; and (3) this notice may not be removed
 -  or altered from any source or modified source distribution.
 *====================================================================*/


#ifndef ARRAY_ACCESS_H
#define ARRAY_ACCESS_H


/*
 *  arrayaccess.h
 *
 *         Pixel access within an array of 32-bit words
 *
 */


#include "environ.h"


    /*---------  1 bit access ---------*/
#define  GET_DATA_BIT(pdata, n) \
    ((*((pdata) + ((n) >> 5)) >> (31 - ((n) & 31))) & 1)

#define  SET_DATA_BIT(pdata, n) \
    (*((pdata) + ((n) >> 5)) |= (0x80000000 >> ((n) & 31)))

#define  CLEAR_DATA_BIT(pdata, n) \
    (*((pdata) + ((n) >> 5)) &= ~(0x80000000 >> ((n) & 31)))


    /*---------  2 bit access ---------*/
#define  GET_DATA_DIBIT(pdata, n)             getDataDibit(pdata, n)
#define  SET_DATA_DIBIT(pdata, n, val)        setDataDibit(pdata, n, val)
#define  CLEAR_DATA_DIBIT(pdata, n)           clearDataDibit(pdata, n)


    /*---------  4 bit access ---------*/
#define  GET_DATA_QBIT(pdata, n)              getDataQbit(pdata, n)
#define  SET_DATA_QBIT(pdata, n, val)         setDataQbit(pdata, n, val)
#define  CLEAR_DATA_QBIT(pdata, n)            clearDataQbit(pdata, n)


    /*---------  8 bit access ---------*/
#ifdef  _BIG_ENDIAN_
#define  GET_DATA_BYTE(pdata, n) \
               (*((UINT8 *)(pdata) + (n)))
#else  /* _LITTLE_ENDIAN_ */
#define  GET_DATA_BYTE(pdata, n) \
               (*(UINT8 *)((UINT32)((UINT8 *)(pdata) + (n)) ^ 3))
#endif  /* _BIG_ENDIAN_ */

#ifdef  _BIG_ENDIAN_
#define  SET_DATA_BYTE(pdata, n, val) \
               (*((UINT8 *)(pdata) + (n)) = (val))
#else  /* _LITTLE_ENDIAN_ */
#define  SET_DATA_BYTE(pdata, n, val) \
               (*(UINT8 *)((UINT32)((UINT8 *)(pdata) + (n)) ^ 3) = (val))
#endif  /* _BIG_ENDIAN_ */


    /*---------  16 bit access ---------*/
#ifdef  _BIG_ENDIAN_
#define  GET_DATA_TWO_BYTES(pdata, n) \
               (*((UINT16 *)(pdata) + (n)))
#else  /* _LITTLE_ENDIAN_ */
#define  GET_DATA_TWO_BYTES(pdata, n) \
               (*(UINT16 *)((UINT32)((UINT16 *)(pdata) + (n)) ^ 2))
#endif  /* _BIG_ENDIAN_ */

#ifdef  _BIG_ENDIAN_
#define  SET_DATA_TWO_BYTES(pdata, n, val) \
               (*((UINT16 *)(pdata) + (n)) = (val))
#else  /* _LITTLE_ENDIAN_ */
#define  SET_DATA_TWO_BYTES(pdata, n, val) \
               (*(UINT16 *)((UINT32)((UINT16 *)(pdata) + (n)) ^ 2) = (val))
#endif  /* _BIG_ENDIAN_ */


#if 0
#define  GET_DATA_BIT(pdata, n)               getDataBit(pdata, n)
#define  SET_DATA_BIT(pdata, n)               setDataBit(pdata, n)
#define  CLEAR_DATA_BIT(pdata, n)             clearDataBit(pdata, n)

#define  GET_DATA_DIBIT(pdata, n)             getDataDibit(pdata, n)
#define  SET_DATA_DIBIT(pdata, n, val)        setDataDibit(pdata, n, val)
#define  CLEAR_DATA_DIBIT(pdata, n)           clearDataDibit(pdata, n)

#define  GET_DATA_QBIT(pdata, n)              getDataQbit(pdata, n)
#define  SET_DATA_QBIT(pdata, n, val)         setDataQbit(pdata, n, val)
#define  CLEAR_DATA_QBIT(pdata, n)            clearDataQbit(pdata, n)

#define  GET_DATA_BYTE(pdata, n)              getDataByte(pdata, n)
#define  SET_DATA_BYTE(pdata, n, val)         setDataByte(pdata, n, val)

#define  GET_DATA_TWO_BYTES(pdata, n)         getDataTwoBytes(pdata, n)
#define  SET_DATA_TWO_BYTES(pdata, n, val)    setDataTwoBytes(pdata, n, val)
#endif


#endif /* ARRAY_ACCESS_H */


