////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		GTMaker.cpp
// File Function:	Implementation of CGTMakerApp class
//					Manipulate file list browser
//
//				Developed by: Yefeng Zheng
//			   First created: Jan, 2001
//			   Last modified: June, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GTMaker.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "GTMakerDoc.h"
#include "GTMakerView.h"
#include "tools.h"

/////////////////////////////////////////////////////////////////////////////
// CGTMakerApp

BEGIN_MESSAGE_MAP(CGTMakerApp, CWinApp)
	//{{AFX_MSG_MAP(CGTMakerApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_CLOSE_ALL, OnFileCloseAll)
	ON_COMMAND(ID_VIEW_NEXT_FILE, OnViewNextFile)
	ON_COMMAND(ID_VIEW_FIRST_FILE, OnViewFirstFile)
	ON_COMMAND(ID_VIEW_LAST_FILE, OnViewLastFile)
	ON_COMMAND(ID_VIEW_PREV_FILE, OnViewPrevFile)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FIRST_FILE, OnUpdateViewFirstFile)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LAST_FILE, OnUpdateViewLastFile)
	ON_UPDATE_COMMAND_UI(ID_VIEW_NEXT_FILE, OnUpdateViewNextFile)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PREV_FILE, OnUpdateViewPrevFile)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGTMakerApp construction

CGTMakerApp::CGTMakerApp()
{
	csFileList = NULL;
	m_nCurFile = -1;
	m_nFile = 0;
	csDir = "" ;
}

CGTMakerApp::~CGTMakerApp()
{
	FreeFileList( );
}
/////////////////////////////////////////////////////////////////////////////
// The one and only CGTMakerApp object

CGTMakerApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CGTMakerApp initialization

BOOL CGTMakerApp::InitInstance()
{
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("Local AppWizard-Generated Applications"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_GTMAKER_TIFF,
		RUNTIME_CLASS(CGTMakerDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CGTMakerView));
	AddDocTemplate(pDocTemplate);

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if( cmdInfo.m_strFileName != "" )
	{
		if (!ProcessShellCommand(cmdInfo))
			return FALSE;
	}
	// The main window has been initialized, so show and update it.
    pMainFrame->ShowWindow(SW_SHOWMAXIMIZED);
	pMainFrame->UpdateWindow();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CGTMakerApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CGTMakerApp message handlers

void CGTMakerApp::OnFileCloseAll() 
{
	CloseAllDocuments(FALSE);
}

/*****************************************************************************
/*	Name:		OnViewNextFile
/*	Function:	Display next file
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnViewNextFile() 
{
	if( csFileList == NULL )
		CreateFileList();
	if( csFileList == NULL || m_nCurFile < 0 || m_nCurFile >= m_nFile)
		return;
	m_nCurFile = GetFileIndex();
	if( m_nCurFile == -1 )
	{
		AfxMessageBox( "Error" );
		return;
	}
	if( m_nCurFile < m_nFile-1 )	m_nCurFile++;
	CloseAllDocuments(FALSE);	
	CString	csFile;
	csFile = csDir + csFileList[m_nCurFile]->GetBuffer(256);
	OpenDocumentFile( csFile );
	csCurFile = csFileList[m_nCurFile]->GetBuffer(256);
}

/*****************************************************************************
/*	Name:		OnUpdateViewNextFile
/*	Function:	Update the menu item "View/Next File"
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnUpdateViewNextFile(CCmdUI* pCmdUI) 
{
	if( m_nCurFile == -1 || m_nCurFile == m_nFile-1 )
		pCmdUI->Enable( FALSE );
	else
		pCmdUI->Enable( TRUE );	
}

/*****************************************************************************
/*	Name:		OnViewPrevFile
/*	Function:	Display previous file
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnViewPrevFile() 
{
	if( csFileList == NULL )
		CreateFileList();
	if( csFileList == NULL || m_nCurFile < 0 || m_nCurFile >= m_nFile)
		return;
	m_nCurFile = GetFileIndex();
	if( m_nCurFile == -1 )
	{
		AfxMessageBox( "Error" );
		return;
	}
	if( m_nCurFile > 0 )	m_nCurFile --;
	CloseAllDocuments(FALSE);
	CString csFile;
	csFile = csDir + csFileList[m_nCurFile]->GetBuffer( 256 ) ;
	OpenDocumentFile( csFile );
	csCurFile = csFileList[m_nCurFile]->GetBuffer(256);
}

/*****************************************************************************
/*	Name:		OnUpdateViewPrevFile
/*	Function:	Update the menu item "View/Prev File"
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnUpdateViewPrevFile(CCmdUI* pCmdUI) 
{
	if( m_nCurFile == -1 || m_nCurFile == 0 )
		pCmdUI->Enable( FALSE );
	else
		pCmdUI->Enable( TRUE );	
}

/*****************************************************************************
/*	Name:		OnViewFirstFile
/*	Function:	Display first file
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnViewFirstFile() 
{
	if( csFileList == NULL )
		CreateFileList();
	if( csFileList == NULL || m_nCurFile < 0 || m_nCurFile >= m_nFile)
		return;
	CloseAllDocuments(FALSE);	
	m_nCurFile = 0;
	CString csFile = csDir + csFileList[m_nCurFile]->GetBuffer( 256 ); 
	OpenDocumentFile( csFile );
	csCurFile = csFileList[m_nCurFile]->GetBuffer(256);
}

/*****************************************************************************
/*	Name:		OnUpdateViewFirstFile
/*	Function:	Update the menu "View/First File"
/*	Parameter:	pCmdUI -- Pointer the menu item
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnUpdateViewFirstFile(CCmdUI* pCmdUI) 
{
	if( m_nCurFile == -1 || m_nCurFile == 0 )
		pCmdUI->Enable( FALSE );
	else
		pCmdUI->Enable( TRUE );
}

/*****************************************************************************
/*	Name:		OnViewLastFile
/*	Function:	Display last file
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnViewLastFile() 
{
	if( csFileList == NULL )
		CreateFileList();
	if( csFileList == NULL || m_nCurFile < 0 || m_nCurFile >= m_nFile)
		return;
	CloseAllDocuments(FALSE);
	CString csFile;
	m_nCurFile = m_nFile-1;
	csFile = csDir + csFileList[m_nCurFile]->GetBuffer( 256 ) ;
	OpenDocumentFile( csFile );
	csCurFile = csFileList[m_nCurFile]->GetBuffer(256);
}

/*****************************************************************************
/*	Name:		OnUpdateViewLastFile
/*	Function:	Update the menu item "View/Last File"
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerApp::OnUpdateViewLastFile(CCmdUI* pCmdUI) 
{
	if( m_nCurFile == -1 || m_nCurFile == m_nFile-1)
		pCmdUI->Enable( FALSE );
	else
		pCmdUI->Enable( TRUE );	
}

/*****************************************************************************
/*	Name:		OpenNewFile
/*	Function:	Open a new file
/*	Parameter:	lpszPathName -- File name
/*	Return:		0 -- Succeed
/*****************************************************************************/
int CGTMakerApp::OpenNewFile(LPCTSTR lpszPathName)
{
	CString Dir = GetPathName( lpszPathName );
	Dir = Dir + "\\";
	if( csDir == "" )
		csDir = Dir;
	else if( csDir != Dir )
	{
		FreeFileList();
		csDir = Dir;
	}

	csCurFile = GetFileName(lpszPathName);
	if( csFileList != NULL )
		m_nCurFile = GetFileIndex();
	else
		m_nCurFile = 1;
	return 0;
}

/*****************************************************************************
/*	Name:		CreateFileList
/*	Function:	Create file list for "Next", "Prev", "First" and "Last" browser
/*	Parameter:	NULL
/*	Return:		0 -- Succeed
/*****************************************************************************/
int CGTMakerApp::CreateFileList()
{
	if( csDir == "" )	return 0;
	char	csSearchName[256];
	WIN32_FIND_DATA  FindFileData;
	HANDLE		hd = NULL;

	sprintf( csSearchName, "%s\\*.tif", csDir );
	hd = FindFirstFile(csSearchName, &FindFileData);
	if( hd == INVALID_HANDLE_VALUE )	return -1;
	
	m_nFile=0;
	m_nMaxFile = 100;
	csFileList = (CString**)malloc( sizeof(CString*)*m_nMaxFile );
	while( 1 )
	{
		csFileList[m_nFile] = new CString( FindFileData.cFileName );
		m_nFile++;
		if( m_nFile >= m_nMaxFile )
		{
			m_nMaxFile += 100;
			csFileList = (CString**)realloc( csFileList, sizeof(CString*)*m_nMaxFile );
		}
		if( FindNextFile(hd, &FindFileData) != TRUE )	
			break;
	}

	for( int i=0; i<m_nFile; i++ )
	{
		if( *csFileList[i] == csCurFile )
		{
			m_nCurFile = i;
			break;
		}
	}
	if( i == m_nFile )
		return -1;
	else
		return 0;
}

/*****************************************************************************
/*	Name:		FreeFileList
/*	Function:	Free file list
/*	Parameter:	NULL
/*	Return:		0 -- Succeed
/*****************************************************************************/
int CGTMakerApp::FreeFileList()
{
	if( csFileList == NULL )	return 0;
	for(int i=0; i<m_nFile; i++ )
		delete csFileList[i];
	free( csFileList );
	csFileList = NULL;
	m_nFile = 0;
	m_nMaxFile = 0;
	m_nCurFile = -1;
	csCurFile = "";
	return 0;
}

/*****************************************************************************
/*	Name:		GetFileIndex
/*	Function:	Get the current file index in the file list
/*	Parameter:	NULL
/*	Return:		Current file index
/*****************************************************************************/
int CGTMakerApp::GetFileIndex()
{
	if( csFileList == NULL )	return -1;

	for( int i=0; i<m_nFile; i++ )
		if( *csFileList[i] == csCurFile )
			return i;
	return -1;
}
