////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		GTMakerDoc.cpp
// File Function:	Implementation of CGTMakerDoc class
//					Open and save segmentation results
//
//				Developed by: Yefeng Zheng
//			   First created: Jan, 2001
//			   Last modified: June, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "tools.h"
#include "GTMaker.h"
#include "GTMakerDoc.h"
#include "GTMakerView.h"
#include <math.h>
#include "define.h"

extern CGTMakerApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CGTMakerDoc

IMPLEMENT_DYNCREATE(CGTMakerDoc, CDocument)

BEGIN_MESSAGE_MAP(CGTMakerDoc, CDocument)
	//{{AFX_MSG_MAP(CGTMakerDoc)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVEAS, OnFileSaveas)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGTMakerDoc construction/destruction

CGTMakerDoc::CGTMakerDoc()
{
	zone = NULL;
	zone_count = 0;

	m_nSelZone = -1;
	m_nSelZoneIndex = NULL;
	m_nSelCorner = -1;
	m_bNeedSave = FALSE;
}

CGTMakerDoc::~CGTMakerDoc()
{
	if( m_nSelZoneIndex != NULL )
		free( m_nSelZoneIndex );
	FreeMem();
}

BOOL CGTMakerDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CGTMakerDoc serialization

void CGTMakerDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
	}
	else
	{
	}
}

/////////////////////////////////////////////////////////////////////////////
// CGTMakerDoc diagnostics

#ifdef _DEBUG
void CGTMakerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CGTMakerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGTMakerDoc commands

/*****************************************************************************
/*	Name:		OnOpenDocument
/*	Function:	Open a image and the corresponding segmentation result file
/*	Parameter:	lpszPathName -- Image file name
/*	Return:		TRUE -- Succeed
/*****************************************************************************/
BOOL CGTMakerDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	
	m_csImgName.Format( "%s", lpszPathName );
	
	CString		csExt = GetFileExtension( lpszPathName );
	csExt.MakeLower();

	if( m_Img.Read( (char*)lpszPathName ) != 0 )
	{
		AfxMessageBox( "Can not read the image!" );
		return FALSE;
	}
	theApp.OpenNewFile( lpszPathName );

	ChangeFileExt( m_csZoneName, (LPTSTR)lpszPathName, "zone" );
	FILE *fp = fopen( m_csZoneName, "rt" );
	if( fp == NULL )	return TRUE;

	char	buf[1024];
	fgets( buf, 1024, fp );
	sscanf( buf, "Total Zones:%d\n", &zone_count );
	zone = (ZONE *) malloc (zone_count * sizeof(ZONE));
  	if( zone == NULL )
	{
		AfxMessageBox( "Memory used up!" );
		return FALSE;
	}

	for( int i=0; i<zone_count; i++ )
	{
		fgets( buf, 1024, fp );
		zone[i].nConf = 0;
		sscanf( buf, "%d%d%d%d%d%lf\n", &zone[i].x, &zone[i].y, 
				&zone[i].w, &zone[i].h, &zone[i].nType, &zone[i].nConf );
		zone[i].nIdType = -1;
	}
	fclose(fp);
	
	m_nSelZoneIndex = (int*)malloc(sizeof(int)*zone_count);
	return TRUE;
}

/*****************************************************************************
/*	Name:		FreeMem
/*	Function:	Free memory used to store segmentation result
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerDoc::FreeMem()
{
	if( zone != NULL )
	{
		free( zone );
		zone = NULL ;
		zone_count = 0;
	}
}

/*****************************************************************************
/*	Name:		OnCloseDocument
/*	Function:	Close document and save modification
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerDoc::OnCloseDocument() 
{
	if( m_bNeedSave == TRUE)
	{
		if( ::MessageBox(NULL, "Do you want to save modified segmentation results?", "Save or not", MB_YESNO|MB_ICONQUESTION) == IDYES)
			OnFileSave();	
	}
	CDocument::OnCloseDocument();
}

/*****************************************************************************
/*	Name:		OnFileSave
/*	Function:	Save modificationi
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerDoc::OnFileSave()
{
	SaveZoneType( );
}

/*****************************************************************************
/*	Name:		OnFileSaveas
/*	Function:	Save modification result to another file
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerDoc::OnFileSaveas()
{
	static char BASED_CODE szFilter[] = "Zone Files (*.zone)|*.zone|All Files (*.*)|*.*||" ;
    CFileDialog dlgSave ( FALSE, "zone", NULL, OFN_ALLOWMULTISELECT | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter );
	if( dlgSave.DoModal() == IDOK)
	{	
		POSITION pos = GetFirstViewPosition( ) ;	
		CGTMakerView* pView = (CGTMakerView*)GetNextView(pos);
		m_csZoneName =  dlgSave.GetPathName();
		SaveZoneType( );
	}
}

/*****************************************************************************
/*	Name:		SaveZoneType
/*	Function:	Save modification result
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerDoc::SaveZoneType( )
{
	FILE *fp = fopen( m_csZoneName, "rt" );
	int nZone, nAverWidth=0, nAverHeight=0, nCharSpace=0, nLineSpace=0;
	if( fp!= NULL )
	{
		fscanf( fp, "Total Zones:%d AverWidth:%d AverHeight:%d CharSpace:%d LineSpace:%d\n", &nZone, &nAverWidth, &nAverHeight, &nCharSpace, &nLineSpace );
		fclose( fp );
	}

	fp = fopen( m_csZoneName, "wt" );
	if( fp == NULL )		return ;
	if( nLineSpace != 0 )
		fprintf( fp, "Total Zones:%5d AverWidth:%d AverHeight:%d CharSpace:%d LineSpace:%d\n", zone_count, nAverWidth, nAverHeight, nCharSpace, nLineSpace );
	else
		fprintf( fp, "Total Zones:%5d\n", zone_count );
	for(int i=0; i<zone_count; i++ )
	{
		fprintf( fp, "%5d%5d%5d%5d%5d\n", zone[i].x, zone[i].y,
				zone[i].w, zone[i].h, zone[i].nType );
	}
	fclose( fp );
	m_bNeedSave = FALSE;
}


/*****************************************************************************
/*	Name:		RemoveSmallZone
/*	Function:	Remove small zones
/*	Parameter:	NULL
/*	Return:		0 -- Succeed
/*****************************************************************************/
int CGTMakerDoc::RemoveSmallZone()
{
	int		i=0;
	BOOL	bRemove;
	while( i<zone_count )
	{
		bRemove = FALSE;
		if( zone[i].w <= 16 && zone[i].h <= 30 )
			bRemove = TRUE;
		else if( zone[i].h < 10 )
			bRemove = TRUE;

		if( bRemove == TRUE )
		{
			for( int j=i; j<zone_count-1; j++ )
				zone[j] = zone[j+1];
			zone_count--;
		}
		else
			i++;
	}
	return 0;
}
