////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		GTMakerDoc.cpp
// File Function:	Implementation of CGTMakerView class
//					Open and save segmentation results
//
//				Developed by: Yefeng Zheng
//			   First created: Jan, 2001
//			   Last modified: June, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "GTMaker.h"
#include "GTMakerDoc.h"
#include "GTMakerView.h"
#include "MainFrm.h"
#include "math.h"
#include "tools.h"
#include "define.h"
#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
// CGTMakerView

IMPLEMENT_DYNCREATE(CGTMakerView, CScrollView)

BEGIN_MESSAGE_MAP(CGTMakerView, CScrollView)
	//{{AFX_MSG_MAP(CGTMakerView)
	ON_COMMAND(ID_ZOOMIN, OnZoomIn)
	ON_COMMAND(ID_ZOOMNORMAL, OnZoomNormal)
	ON_COMMAND(ID_ZOOMOUT, OnZoomOut)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_EDIT_ADD_ZONE, OnEditAddZone)
	ON_UPDATE_COMMAND_UI(ID_EDIT_ADD_ZONE, OnUpdateEditAddZone)
	ON_COMMAND(ID_EDIT_DELETE_ZONE, OnEditDeleteZone)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DELETE_ZONE, OnUpdateEditDeleteZone)
	ON_COMMAND(ID_ZOOMFIT, OnZoomFit)
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_VIEW_ZONE_CONF, OnViewZoneConf)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZONE_CONF, OnUpdateViewZoneConf)
	ON_COMMAND(ID_VIEW_ZONE, OnViewZone)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ZONE, OnUpdateViewZone)
	ON_COMMAND(ID_ZONE_HAND, OnZoneHand)
	ON_UPDATE_COMMAND_UI(ID_ZONE_HAND, OnUpdateZoneHand)
	ON_COMMAND(ID_ZONE_NOISE, OnZoneNoise)
	ON_UPDATE_COMMAND_UI(ID_ZONE_NOISE, OnUpdateZoneNoise)
	ON_COMMAND(ID_ZONE_PRINT, OnZonePrint)
	ON_UPDATE_COMMAND_UI(ID_ZONE_PRINT, OnUpdateZonePrint)
	ON_COMMAND(ID_VIEW_IMAGE, OnViewImage)
	ON_UPDATE_COMMAND_UI(ID_VIEW_IMAGE, OnUpdateViewImage)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGTMakerView construction/destruction

CGTMakerView::CGTMakerView()
{
	m_nRatio = 1;
	m_ViewStatus = VIEW_ZONE;
	LastPoint = CPoint( -1, -1 );
	CurrentPoint = CPoint( -1, -1 );
}

CGTMakerView::~CGTMakerView()
{
}

BOOL CGTMakerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGTMakerView drawing

/*****************************************************************************
/*	Name:		OnDraw
/*	Function:	Draw segmentation result on screen
/*	Parameter:	pDC -- CDC pointer
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnDraw(CDC* pDC)
{
//Dispaly the original image
	int		PEN_WIDTH = 2;
	RECT	rc;
	CGTMakerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	int w = pDoc->m_Img.GetWidth();
	int h = pDoc->m_Img.GetHeight();
	SetScrollSizes(MM_TEXT, CSize((int)(w*m_nRatio), (int)(h*m_nRatio) ) ) ;
	pDoc->m_Img.ShowStretchImgDIB (pDC->GetSafeHdc(), 0, 0, (int)(pDoc->m_Img.GetWidth()*m_nRatio), (int)(pDoc->m_Img.GetHeight()*m_nRatio) ) ;
	
	if( m_ViewStatus == VIEW_IMAGE_ONLY )		return;

//Display the adding zone
	if( ( m_ViewStatus & VIEW_ADDING ) != 0 )
	{
		if( LastPoint.x >= 0 && LastPoint.y >= 0 && CurrentPoint.x >= 0 && CurrentPoint.y >= 0 )
		{
			CPen   *pGreenPen = new CPen( PS_SOLID, PEN_WIDTH, RGB(0, 255, 0) );
			CPen   *pOldPen = pDC->SelectObject( pGreenPen );
			pDC->MoveTo( (int)(LastPoint.x*m_nRatio), (int)(LastPoint.y*m_nRatio) );
			pDC->LineTo( (int)(CurrentPoint.x*m_nRatio), (int)(LastPoint.y*m_nRatio) );
			pDC->LineTo( (int)(CurrentPoint.x*m_nRatio), (int)(CurrentPoint.y*m_nRatio) );
			pDC->LineTo( (int)(LastPoint.x*m_nRatio), (int)(CurrentPoint.y*m_nRatio) );
			pDC->LineTo( (int)(LastPoint.x*m_nRatio), (int)(LastPoint.y*m_nRatio) );
			pDC->SelectObject( pOldPen );
			delete pGreenPen;
		}
	}
	else
	{
		if( LastPoint.x >= 0 && LastPoint.y >= 0 && CurrentPoint.x >= 0 && CurrentPoint.y >= 0 )
		{
			CPen   *pPen = new CPen( PS_DOT, 1, RGB(0, 0, 0) );
			CPen   *pOldPen = pDC->SelectObject( pPen );
			pDC->MoveTo( (int)(LastPoint.x*m_nRatio), (int)(LastPoint.y*m_nRatio) );
			pDC->LineTo( (int)(CurrentPoint.x*m_nRatio), (int)(LastPoint.y*m_nRatio) );
			pDC->LineTo( (int)(CurrentPoint.x*m_nRatio), (int)(CurrentPoint.y*m_nRatio) );
			pDC->LineTo( (int)(LastPoint.x*m_nRatio), (int)(CurrentPoint.y*m_nRatio) );
			pDC->LineTo( (int)(LastPoint.x*m_nRatio), (int)(LastPoint.y*m_nRatio) );
			pDC->SelectObject( pOldPen );
			delete pPen;
		}
	}

//Get zones already added or loaded from files
	int zone_count = pDoc->zone_count;
	ZONE	*zone = pDoc->zone;
	if( zone_count <= 0 || zone == NULL )	
		return;

	if( (m_ViewStatus & VIEW_ZONE) != 0 )
	{
			CPen   *pYellowPen = new CPen( PS_SOLID, PEN_WIDTH, RGB(255, 255, 0) ); 
			CPen   *pBluePen = new CPen( PS_SOLID, PEN_WIDTH, RGB(0, 0, 255) );
			CPen   *pRedPen = new CPen( PS_SOLID, PEN_WIDTH, RGB(255, 0, 0) );
			CPen   *pGreenPen = new CPen( PS_SOLID, PEN_WIDTH, RGB(0, 255, 0) ); 
			CPen   *pOldPen = (CPen*)pDC->SelectObject( pRedPen );

			for( int i=0; i<zone_count; i++ )
			{
				switch( zone[i].nType )
				{
				case TYPE_HANDWRITING:
					pDC->SelectObject( pRedPen );
					break;
				case TYPE_PRINT_TEXT:
					pDC->SelectObject( pBluePen );
					break;
				case TYPE_NOISE:
					pDC->SelectObject( pGreenPen );
					break;
				case TYPE_OTHERS:
					pDC->SelectObject( pYellowPen );
					break;
				default:
					pDC->SelectObject( pYellowPen );
					break;
				}

				rc.left = (long)(zone[i].x*m_nRatio);
				rc.top = (long)(zone[i].y*m_nRatio);
				rc.right = (long)((zone[i].x+zone[i].w)*m_nRatio);
				rc.bottom = (long)((zone[i].y+zone[i].h)*m_nRatio);
				pDC->MoveTo( rc.left, rc.top );
				pDC->LineTo( rc.right, rc.top );
				pDC->LineTo( rc.right, rc.bottom );
				pDC->LineTo( rc.left, rc.bottom );
				pDC->LineTo( rc.left, rc.top );
			}
			pDC->SelectObject( pOldPen );
			delete	pBluePen;
			delete  pRedPen;
			delete	pGreenPen;
			delete  pYellowPen;
	}
	if( (m_ViewStatus & VIEW_ZONECONF ) != 0 )
	{
			char	text[256];
			for( int i=0; i<zone_count; i++ )
			{
				rc.left = (long)(zone[i].x*m_nRatio);
				rc.top = (long)(zone[i].y*m_nRatio);
				strcpy( text, "" );
				if( (m_ViewStatus & VIEW_ZONECONF ) != 0 )
					sprintf( text, "%3.2f", zone[i].nConf );
				
				pDC->SetTextColor( RGB( 255, 0, 0 ) );
				pDC->TextOut( rc.left, rc.top, text );				
			}
	}

//Display selected zones
	if( pDoc->m_nSelZone >0 )
	{
		CPen   *pYellowPen = new CPen( PS_SOLID, PEN_WIDTH, RGB(255, 255, 0) );
		CPen   *pOldPen = pDC->SelectObject( pYellowPen );
		for( int i=0; i<pDoc->m_nSelZone; i++ )
		{
			ZONE	box = pDoc->zone[pDoc->m_nSelZoneIndex[i]];
			box.x = (short)(box.x*m_nRatio);
			box.y = (short)(box.y*m_nRatio);
			box.w = (short)(box.w*m_nRatio);
			box.h = (short)(box.h*m_nRatio);

			pDC->MoveTo( box.x, box.y );
			pDC->LineTo( box.x+box.w, box.y );
			pDC->LineTo( box.x+box.w, box.y+box.h );
			pDC->LineTo( box.x, box.y+box.h );
			pDC->LineTo( box.x, box.y );

			pDC->FillSolidRect( box.x-3, box.y-3, 6, 6, RGB(0, 0, 0) ); 
			pDC->FillSolidRect( box.x+box.w-3, box.y-3, 6, 6, RGB(0, 0, 0) ); 
			pDC->FillSolidRect( box.x-3, box.y+box.h-3, 6, 6, RGB(0, 0, 0) ); 
			pDC->FillSolidRect( box.x+box.w-3, box.y+box.h-3, 6, 6, RGB(0, 0, 0) ); 
		}
		pDC->SelectObject( pOldPen );
		delete pYellowPen;
	}
}

/*****************************************************************************
/*	Name:		OnInitialUpdate
/*	Function:	Initial window size
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	CSize sizeTotal;
	sizeTotal.cx = sizeTotal.cy = 100;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

/////////////////////////////////////////////////////////////////////////////
// CGTMakerView printing

BOOL CGTMakerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	return DoPreparePrinting(pInfo);
}

void CGTMakerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CGTMakerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CGTMakerView diagnostics

#ifdef _DEBUG
void CGTMakerView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CGTMakerView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CGTMakerDoc* CGTMakerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGTMakerDoc)));
	return (CGTMakerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGTMakerView message handlers

////////////////////////////////////////////////////////////////////////////////
// Adjust display size
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
/*	Name:		OnZoomIn
/*	Function:	Increase the display image size by two times
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZoomIn() 
{
	m_nRatio *=2;
	Invalidate(FALSE);	
}

/*****************************************************************************
/*	Name:		OnZoomNormal
/*	Function:	1:1 display size
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZoomNormal() 
{
	m_nRatio = 1;
	Invalidate(FALSE);
}

/*****************************************************************************
/*	Name:		OnZoomOut
/*	Function:	Decrease the display image size by two times
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZoomOut() 
{
	m_nRatio /=2;
	Invalidate(FALSE);	
}

/*****************************************************************************
/*	Name:		OnZoomFit
/*	Function:	Adjust the display size to fit current window size
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZoomFit() 
{
	CGTMakerDoc *pDoc = (CGTMakerDoc*)GetDocument();
	int ImgW = pDoc->m_Img.GetWidth();
	int ImgH = pDoc->m_Img.GetHeight();
	RECT rcWin;
	GetClientRect( &rcWin );
	SetScrollSizes(MM_TEXT, CSize(rcWin.right-rcWin.left, rcWin.bottom-rcWin.top ) ) ;
	GetClientRect( &rcWin );
	m_nRatio = (rcWin.right - rcWin.left) * 1.0 / ImgW;
	m_nRatio = min( m_nRatio, (rcWin.bottom - rcWin.top) * 1.0 / ImgH);
	Invalidate(FALSE);
}

/*****************************************************************************
/*	Name:		OnViewZoneConf
/*	Function:	Display classification confidence
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnViewZoneConf() 
{
	if( (m_ViewStatus & VIEW_ZONECONF) == 0 )
	{
		if( m_ViewStatus == VIEW_IMAGE_ONLY )
			m_ViewStatus = VIEW_ZONE;
		m_ViewStatus |= VIEW_ZONECONF;
	}
	else
		m_ViewStatus &= (~VIEW_ZONECONF);
	Invalidate(FALSE);			
}

/*****************************************************************************
/*	Name:		OnUpdateViewZoneConf
/*	Function:	Update menu item "View/Zone Conf"
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateViewZoneConf(CCmdUI* pCmdUI) 
{
	if( (m_ViewStatus & VIEW_ZONECONF ) == 0 )
		pCmdUI->SetCheck( 0 );
	else
		pCmdUI->SetCheck( 1 );		
}

/*****************************************************************************
/*	Name:		OnViewZone
/*	Function:	Display zones
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnViewZone() 
{
	if( (m_ViewStatus & VIEW_ZONE) == 0 )
	{
		if( m_ViewStatus == VIEW_IMAGE_ONLY )
			m_ViewStatus = VIEW_ZONE;
		else
			m_ViewStatus |= VIEW_ZONE;
	}
	else
		m_ViewStatus &= (~VIEW_ZONE);
	Invalidate(FALSE);				
} 

/*****************************************************************************
/*	Name:		OnUpdateViewZone
/*	Function:	Update menu item "View/View Zone"
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateViewZone(CCmdUI* pCmdUI) 
{
	if( (m_ViewStatus & VIEW_ZONE ) == 0 )
		pCmdUI->SetCheck( 0 );
	else
		pCmdUI->SetCheck( 1 );			
}

/*****************************************************************************
/*	Name:		OnViewImage
/*	Function:	Display image only
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnViewImage() 
{
	if( m_ViewStatus == VIEW_IMAGE_ONLY )
		m_ViewStatus = VIEW_ZONE;
	else
		m_ViewStatus = VIEW_IMAGE_ONLY;
	Invalidate(FALSE);					
}

/*****************************************************************************
/*	Name:		OnUpdateViewImage
/*	Function:	Update menu item "View/View Image Only"
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateViewImage(CCmdUI* pCmdUI) 
{
	if( (m_ViewStatus & VIEW_IMAGE_ONLY ) == 0 )
		pCmdUI->SetCheck( 0 );
	else
		pCmdUI->SetCheck( 1 );	
}

////////////////////////////////////////////////////////////////////////////////
// Mouse message handlers
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
/*	Name:		OnMoveMove
/*	Function:	Display mouse position and add new zones
/*	Parameter:	nFlags -- Flag indicate other status
/*				point  -- Mouse position
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnMouseMove(UINT nFlags, CPoint point) 
{
	m_MousePos = point;
	CPoint PntOffSet = GetScrollPosition( );
	point.x += PntOffSet.x;
	point.y += PntOffSet.y;
	point.x = (int)(point.x/m_nRatio);
	point.y = (int)(point.y/m_nRatio);
	CMainFrame*   pMainFrm = (CMainFrame*)(AfxGetApp()->m_pMainWnd);
	pMainFrm->SetXY(point);
	
	if( (nFlags & MK_LBUTTON ) == 0 )	
	{
		CScrollView::OnMouseMove(nFlags, point);
		return;
	}
	if( (m_ViewStatus & VIEW_ADDING) != 0 )
	{//Adding zone
		CurrentPoint = point;
		Invalidate( FALSE );
		CScrollView::OnMouseMove(nFlags, point);
		return;
	}
//Modifying zone
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	return;
	if( pDoc->m_nSelZone == 0 )
	{
		CurrentPoint = point;
		Invalidate( FALSE );
		CScrollView::OnMouseMove(nFlags, point);
		return;
	}
	if( pDoc->m_nSelZone == 1 )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[0]];
		if( pDoc->m_nSelCorner < 0 || pDoc->m_nSelCorner >= 4 )
		{//Move the whole zone
			pSelZone->x += (short)(point.x - LastPoint.x);
			pSelZone->y += (short)(point.y - LastPoint.y);
			pDoc->m_bNeedSave = TRUE;
		}
		else
		{//Move one corner of the zone
			switch( pDoc->m_nSelCorner )
			{
			case 0:
				pSelZone->x += (short)(point.x - LastPoint.x);
				pSelZone->y += (short)(point.y - LastPoint.y);
				pSelZone->w -= (short)(point.x - LastPoint.x);
				pSelZone->h -= (short)(point.y - LastPoint.y);
				break;
			case 1:
				pSelZone->y += (short)(point.y - LastPoint.y);
				pSelZone->h -= (short)(point.y - LastPoint.y);
				pSelZone->w += (short)(point.x - LastPoint.x);
				break;
			case 2:
				pSelZone->w += (short)(point.x - LastPoint.x);
				pSelZone->h += (short)(point.y - LastPoint.y);
				break;
			case 3:
				pSelZone->x += (short)(point.x - LastPoint.x);
				pSelZone->w -= (short)(point.x - LastPoint.x);
				pSelZone->h += (short)(point.y - LastPoint.y);
				break;
			}
			if( pSelZone->w < 0 )
			{
				pSelZone->w = -pSelZone->w;
				pSelZone->x -= pSelZone->w;
				pDoc->m_nSelCorner = GetSelCorner( *pSelZone, point);
			}
			if( pSelZone->h < 0 )
			{
				pSelZone->h = -pSelZone->h;
				pSelZone->y -= pSelZone->h;
				pDoc->m_nSelCorner = GetSelCorner( *pSelZone, point);
			}
			pDoc->m_bNeedSave = TRUE;
		}
	}
	LastPoint = point;
	Invalidate( FALSE );
}

/*****************************************************************************
/*	Name:		OnLButtonDown
/*	Function:	Select one zone
/*	Parameter:	nFlags -- Flags to indicate other status
/*				point  -- Mouse position
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CGTMakerDoc	*pDoc = GetDocument();
	
	CPoint	OffsetPos = GetScrollPosition( );
	point.x = (long)( (point.x + OffsetPos.x)/m_nRatio );
	point.y = (long)( (point.y + OffsetPos.y)/m_nRatio );
	
	if( (m_ViewStatus & VIEW_ADDING) == 0 )//Select zone
	{
		int		m_nOldSelZone = pDoc->m_nSelZone;
		pDoc->m_nSelZone = 0;
		ZONE *zone = pDoc->zone;
		SelectZone( point );
		if( pDoc->m_nSelZone == 0 && m_nOldSelZone == 1 ) 
		{
			int nSelCorner = GetSelCorner( pDoc->zone[m_nOldSelZone], point );
			if( nSelCorner != -1 )
			{
				pDoc->m_nSelZone = m_nOldSelZone;
				pDoc->m_nSelCorner = nSelCorner;
			}
		}
		else if( pDoc->m_nSelZone == 1 )
			pDoc->m_nSelCorner = GetSelCorner( pDoc->zone[pDoc->m_nSelZoneIndex[0]], point );

		Invalidate( FALSE );
	}
	LastPoint = point;
	CScrollView::OnLButtonDown(nFlags, point);	
}

/*****************************************************************************
/*	Name:		OnLButtonUp
/*	Function:	Add a new vertex for a new zone
/*	Parameter:	nFlags -- Flags indicate other status
/*				point  -- Mouse position
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if( (m_ViewStatus & VIEW_ADDING) != 0 )//Select zone
	{
		if( !(LastPoint.x < 0 || LastPoint.y < 0 || CurrentPoint.x < 0 || CurrentPoint.y < 0 ||
			fabs( CurrentPoint.x-LastPoint.x) < 10 || fabs( CurrentPoint.y-LastPoint.y) < 10 ) )
		{
			CGTMakerDoc *pDoc = GetDocument();
			pDoc->zone_count++;
			pDoc->zone = (ZONE*)realloc( pDoc->zone, sizeof(ZONE)*pDoc->zone_count );
			pDoc->zone[pDoc->zone_count-1].x = (short)min( LastPoint.x, CurrentPoint.x);
			pDoc->zone[pDoc->zone_count-1].y = (short)min( LastPoint.y, CurrentPoint.y);
			pDoc->zone[pDoc->zone_count-1].w = (short)fabs( CurrentPoint.x - LastPoint.x );
			pDoc->zone[pDoc->zone_count-1].h = (short)fabs( CurrentPoint.y - LastPoint.y );
			pDoc->zone[pDoc->zone_count-1].nType = TYPE_OTHERS;
			pDoc->m_nSelZone = 1;
			pDoc->m_nSelZoneIndex[0] = pDoc->zone_count-1;
			pDoc->m_bNeedSave = TRUE;
		}
	}
	else
	{
		if( CurrentPoint != CPoint( -1, -1 ) )
			SelectZone( CurrentPoint, LastPoint );
	}

	LastPoint = CPoint( -1, -1 );
	CurrentPoint = CPoint( -1, -1 );
	Invalidate( FALSE );
	CScrollView::OnLButtonUp(nFlags, point);
}

/*****************************************************************************
/*	Name:		OnRButtonDown
/*	Function:	Display a dialog for editting zone type
/*	Parameter:	nFlags -- Flags indicate other status
/*				point  -- Mouse position
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CGTMakerDoc* pDoc = GetDocument();
	if( pDoc == NULL )		return;
	
	CPoint	OffsetPos = GetScrollPosition( );
	point.x = (long)( (point.x + OffsetPos.x)/m_nRatio );
	point.y = (long)( (point.y + OffsetPos.y)/m_nRatio );
	
	pDoc->m_nSelZone = -1;
	SelectZone( point );
}

/*****************************************************************************
/*	Name:		OnKeyDown
/*	Function:	Use key to browse the image and delete a zone
/*	Parameter:	nChar   -- Specifies the virtual-key code of the given key
/*				nRepCnt -- Repeat count (the number of times the keystroke is 
/*						   repeated as a result of the user holding down the key).
/*				nFlags  -- Flags indicate other status
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
    switch ( nChar )
    {
        case VK_HOME :
            SendMessage ( WM_HSCROLL, SB_LEFT, 0 ) ;
            break ;
        case VK_END :
            SendMessage ( WM_HSCROLL, SB_RIGHT, 0 ) ;
            break ;
        case VK_PRIOR :
            SendMessage ( WM_VSCROLL, SB_PAGEUP, 0 ) ;
            break ;
        case VK_NEXT :
            SendMessage ( WM_VSCROLL, SB_PAGEDOWN, 0 ) ;
            break ;
        case VK_UP :
            SendMessage ( WM_VSCROLL, SB_LINEUP, 0 ) ;
            break ;
        case VK_DOWN :
            SendMessage ( WM_VSCROLL, SB_LINEDOWN, 0 ) ;
            break ;
        case VK_LEFT :
            SendMessage ( WM_HSCROLL, SB_LINELEFT, 0 ) ;
            break ;
        case VK_RIGHT :
            SendMessage ( WM_HSCROLL, SB_LINERIGHT, 0 ) ;
            break ;
		case VK_DELETE:
			OnEditDeleteZone();
			break;
    }
	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
}

///////////////////////////////////////////////////////////////////////////////
// Modify zone segmentation information
///////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
/*	Name:		OnEditAddZone
/*	Function:	Add a new zone
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnEditAddZone() 
{
	if( (m_ViewStatus & VIEW_ADDING) == 0 )
		m_ViewStatus |= VIEW_ADDING;
	else
		m_ViewStatus &= (~VIEW_ADDING);
}

/*****************************************************************************
/*	Name:		OnUpdateEditAddZone
/*	Function:	Update the menu item "Edit/Add Zone"
/*	Parameter:	pCmdUI -- Pointer to menu item
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateEditAddZone(CCmdUI* pCmdUI) 
{
	CGTMakerDoc* pDoc = GetDocument();
	if( pDoc == NULL )		return;

	if( (m_ViewStatus & VIEW_ADDING) == 0 )
		pCmdUI->SetCheck( 0 );
	else
		pCmdUI->SetCheck( 1 );
}

/*****************************************************************************
/*	Name:		OnEditDeleteZone
/*	Function:	Delete a zone
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnEditDeleteZone()
{
	CGTMakerDoc* pDoc = GetDocument();
	if( pDoc == NULL )		
		return;
	if( pDoc->m_nSelZone == 0 )	
		return;

	ZONE *pZone = pDoc->zone;
	int	 m_nSelZone = pDoc->m_nSelZone;
	int  *m_nSelZoneIndex = pDoc->m_nSelZoneIndex;
	int i;
	for( i=0; i<pDoc->zone_count; i++ )
		pZone[i].nSelect = FALSE;
	for( i=0; i<m_nSelZone; i++ )
		pZone[m_nSelZoneIndex[i]].nSelect = TRUE;

	i=0;
	while( i<pDoc->zone_count )
	{
		if( pZone[i].nSelect == TRUE )
		{
			for( int j=i; j<pDoc->zone_count-1; j++ )
				pZone[j] = pZone[j+1];
			pDoc->zone_count--;
		}
		else
			i++;
	}

	pDoc->m_nSelZone = 0;
	pDoc->m_bNeedSave = TRUE;
	Invalidate( FALSE );
}

/*****************************************************************************
/*	Name:		OnUpdateEditDeleteZone
/*	Function:	Update status of the menu item "Edit/Delete Zone"
/*	Parameter:	pCmdUI -- Pointer to the menu item
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateEditDeleteZone(CCmdUI* pCmdUI) 
{
	CGTMakerDoc* pDoc = GetDocument();
	if( pDoc == NULL )		return;
	if( pDoc->m_nSelZone <0 || pDoc->m_nSelZone >= pDoc->zone_count )
		pCmdUI->Enable( FALSE );
	else
		pCmdUI->Enable( TRUE );
}

/*****************************************************************************
/*	Name:		GetSelCorner
/*	Function:	Get the selected corner of a zone
/*	Parameter:	box   -- Zone currently selected
/*				point -- Mouse position
/*	Return:		Corner index selected
/*****************************************************************************/
int CGTMakerView::GetSelCorner(ZONE box, CPoint point)
{
	int nSelCorner = -1;
	int nMinDist = 1000;
	int nMin = -1;
	int Dist = (int)GetDistance( CPoint( box.x, box.y), point);
	if( Dist < nMinDist )
	{
		nMinDist = Dist;
		nMin = 0;
	}
	Dist = (int)GetDistance( CPoint( box.x+box.w, box.y), point );
	if( Dist < nMinDist )
	{
		nMinDist = Dist;
		nMin = 1;
	}
	Dist = (int)GetDistance( CPoint( box.x+box.w, box.y+box.h), point );
	if( Dist < nMinDist )
	{
		nMinDist = Dist;
		nMin = 2;
	}
	Dist = (int)GetDistance( CPoint( box.x, box.y+box.h), point );
	if( Dist < nMinDist )
	{
		nMinDist = Dist;
		nMin = 3;
	}
	if( nMinDist >= 20 )
		return -1;
	else
		return nMin;
}
	
/*****************************************************************************
/*	Name:		SelectZone
/*	Function:	Get current selected zone
/*	Parameter:	point -- Mouse position
/*	Return:		Zone index currently selected
/*****************************************************************************/
int CGTMakerView::SelectZone(CPoint point)
{
	CGTMakerDoc *pDoc = GetDocument();
	pDoc->m_nSelZone = 0;
	ZONE *zone = pDoc->zone;
	int  zone_count = pDoc->zone_count;
	double  nMinDist = 10e+5;
	for( int i=0; i<zone_count; i++ )
	{
		if( point.x > zone[i].x && point.x < zone[i].x+zone[i].w &&
			point.y > zone[i].y && point.y < zone[i].y+zone[i].h )
		{
			double nDist = GetDistance( CPoint(zone[i].x, zone[i].y), point);
			nDist = min( nDist, GetDistance( CPoint( zone[i].x+zone[i].w, zone[i].y), point) );
			nDist = min( nDist, GetDistance( CPoint( zone[i].x+zone[i].w, zone[i].y+zone[i].h), point));
			nDist = min( nDist, GetDistance( CPoint( zone[i].x, zone[i].y+zone[i].h), point) );
			if( nDist < nMinDist )
			{
				pDoc->m_nSelZone = 1;
				pDoc->m_nSelZoneIndex[0] = i;
				nMinDist = nDist;
			}
		}
	}
	return 0;
}

/*****************************************************************************
/*	Name:		SelectZone
/*	Function:	Select all zone inside the region (pt1, pt2)
/*	Parameter:	pt1 -- Top left corner of the region
/*				pt2 -- Bottom right corner of the region
/*	Return:		0 -- Succeed
/*****************************************************************************/
int CGTMakerView::SelectZone(CPoint pt1, CPoint pt2 )
{
	CRect rc;
	rc.left = min( pt1.x, pt2.x );
	rc.top = min( pt1.y, pt2.y );
	rc.right = max( pt1.x, pt2.x );
	rc.bottom = max( pt1.y, pt2.y );

	CGTMakerDoc *pDoc = GetDocument();
	pDoc->m_nSelZone = 0;
	ZONE *zone = pDoc->zone;
	int  zone_count = pDoc->zone_count;
	double  nMinDist = 10e+5;
	for( int i=0; i<zone_count; i++ )
	{
		if( !rc.PtInRect( CPoint( zone[i].x, zone[i].y) ) )
			continue;
		if( !rc.PtInRect( CPoint( zone[i].x, zone[i].y+zone[i].h ) ) )
			continue;
		if( !rc.PtInRect( CPoint( zone[i].x+zone[i].w, zone[i].y ) ) )
			continue;
		if( !rc.PtInRect( CPoint( zone[i].x+zone[i].w, zone[i].y+zone[i].h) ) )
			continue;
		pDoc->m_nSelZoneIndex[pDoc->m_nSelZone] = i;
		pDoc->m_nSelZone++;
	}
	return 0;
}

/*****************************************************************************
/*	Name:		OnZoneHand
/*	Function:	Change the type of currently selected zones to handwriting
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZoneHand() 
{
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	
		return ;
	
	for( int i=0; i<pDoc->m_nSelZone; i++ )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[i]];
		if( pSelZone->nType != TYPE_HANDWRITING )
			pDoc->m_bNeedSave = TRUE;
		pSelZone->nType = TYPE_HANDWRITING;
	}
}

/*****************************************************************************
/*	Name:		OnUpdateZoneHand
/*	Function:	Only the type of all selected zones are handwriting, the button
/*				of TYPE_HANDWRITING will be selected.
/*	Parameter:	
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateZoneHand(CCmdUI* pCmdUI) 
{
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	return ;

	if( pDoc->m_nSelZone > 0 )
		pCmdUI->Enable( TRUE );
	else
		pCmdUI->Enable( FALSE );
	
	for( int i=0; i<pDoc->m_nSelZone; i++ )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[i]];
		if( pSelZone->nType != TYPE_HANDWRITING )
			break;
	}
	if( i>=pDoc->m_nSelZone && pDoc->m_nSelZone > 0 )
		pCmdUI->SetCheck( TRUE );
	else
		pCmdUI->SetCheck( FALSE );
}

/*****************************************************************************
/*	Name:		OnZoneNoise
/*	Function:	Change the type of currently selected zones to noise
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZoneNoise() 
{
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	
		return ;

	for( int i=0; i<pDoc->m_nSelZone; i++ )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[i]];
		if( pSelZone->nType != TYPE_NOISE )
			pDoc->m_bNeedSave = TRUE;

		pSelZone->nType = TYPE_NOISE;	
	}
}

/*****************************************************************************
/*	Name:		OnUpdateZoneNoise
/*	Function:	Only the type of all selected zones are noise, the button
/*				of TYPE_NOISE will be selected.
/*	Parameter:	
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateZoneNoise(CCmdUI* pCmdUI) 
{
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	return ;

	if( pDoc->m_nSelZone > 0 )
		pCmdUI->Enable( TRUE );
	else
		pCmdUI->Enable( FALSE );
	
	for( int i=0; i<pDoc->m_nSelZone; i++ )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[i]];
		if( pSelZone->nType != TYPE_NOISE )
			break;
	}
	if( (i>=pDoc->m_nSelZone) & (pDoc->m_nSelZone > 0) )
		pCmdUI->SetCheck( TRUE );
	else
		pCmdUI->SetCheck( FALSE );
	
}

/*****************************************************************************
/*	Name:		OnZonePrint
/*	Function:	Change the type of currently selected zones to machine printed text
/*	Parameter:	NULL
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnZonePrint() 
{
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	
		return ;

	for( int i=0; i<pDoc->m_nSelZone; i++ )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[i]];
		if( pSelZone->nType != TYPE_PRINT_TEXT )
			pDoc->m_bNeedSave = TRUE;
		pSelZone->nType = TYPE_PRINT_TEXT;	
	}	
}

/*****************************************************************************
/*	Name:		OnUpdateZonePrint
/*	Function:	Only the type of all selected zones are machine printed text,
/*				the button of TYPE_PRINT_TEXT will be selected.
/*	Parameter:	
/*	Return:		NULL
/*****************************************************************************/
void CGTMakerView::OnUpdateZonePrint(CCmdUI* pCmdUI) 
{
	CGTMakerDoc *pDoc = GetDocument();
	if( pDoc == NULL )	return ;

	if( pDoc->m_nSelZone > 0 )
		pCmdUI->Enable( TRUE );
	else
		pCmdUI->Enable( FALSE );
	
	for( int i=0; i<pDoc->m_nSelZone; i++ )
	{
		ZONE *pSelZone = &pDoc->zone[pDoc->m_nSelZoneIndex[i]];
		if( pSelZone->nType != TYPE_PRINT_TEXT )
			break;
	}
	if( (i>= pDoc->m_nSelZone) & (pDoc->m_nSelZone > 0) )
		pCmdUI->SetCheck( TRUE );
	else
		pCmdUI->SetCheck( FALSE );	
}

