#include "stdafx.h"
#include "class_ba.h"

CImg::CImg()
{
    dataposition = 0;
    linenumber = 0;
    filehandle = m_hFile;
    memoryhandle = NULL;
}


CImg::CImg(const char *pszFileName,UINT nOpenFlags)
                                    :CFile(pszFileName,nOpenFlags)
{       
    int i,j;
    
    lstrcpy((LPSTR)imagefilename,(LPSTR)pszFileName);
    dataposition = 0;
    linenumber = 0;
    filehandle = m_hFile;
    openflags = nOpenFlags;
    memoryhandle = NULL;
    errorflag = FALSE;
    errormessage[0] = 0;
    j = 0;
    for (i=0;i<sizeof(imagefilename);i++)
    {   
        imagefilename[j++] = imagefilename[i];
        if(imagefilename[i]==0) break;
    }
}


CImg::~CImg()
{
}


BOOL CImg::openimagefile(char *filename,UINT nOpenFlags)
{
    int i,j;
    
    if (CFile::Open(filename,nOpenFlags))
    {
        filehandle = m_hFile;
        memoryhandle = NULL;
        lstrcpy((LPSTR)imagefilename,(LPSTR)filename);
        dataposition = 0;
        linenumber = 0;
        openflags = nOpenFlags;
        errorflag = FALSE;
        errormessage[0] = 0;
        j = 0;
        for (i=0;i<sizeof(imagefilename);i++)
        {   
            if(imagefilename[i]==' ') continue;
            imagefilename[j++] = imagefilename[i];
            if(imagefilename[i]==0) break;
        }

        return TRUE;      
    }   
    else     
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage,"Can't open file .");
        return FALSE;
    }

}


void CImg::closeimagefile()
{
    CFile::Close();
    filehandle = m_hFile;
    dataposition = 0;
    linenumber = 0;
    *imagefilename = 0;
    return;
}


long CImg::imagemetrics()
{      
    long metrics = (long)height << 16 ;
    metrics += width;
    return metrics;
}


int CImg::imageinfor()
{
    int infor;
    infor = 0;
    if (filehandle != CFile::hFileNull) infor = 1;
    if (valid) infor += 2;
    infor += depth << 2;
    infor += color << 8;
    infor += compression << 10;
    return (infor);
}


BOOL CImg::isvalidfile(void)
{
    char extemp[4];
    int i;
    
    if(filehandle==CFile::hFileNull)
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage,"No file opened .");
        return FALSE;
    }
    
    i=lstrlen((LPCSTR)imagefilename);
    if(imagefilename[i-4] != '.')
    {
//        errorflag = TRUE;
//        lstrcpy((LPSTR)errormessage,"file name needs extension .");
        return FALSE;
    }
    
    extemp[0]=imagefilename[i-3];
    extemp[1]=imagefilename[i-2];
    extemp[2]=imagefilename[i-1];
    extemp[3]=0;
    if(lstrcmpi((LPCSTR)filextension,(LPCSTR)extemp)==0) return TRUE;
    else
    {
//        errorflag = TRUE;
//        lstrcpy((LPSTR)errormessage,"File extension not valid .");
        return FALSE;
    }
}


BOOL CImg::writemyhead(HANDLE handle)
{
    MYHEAD myhead;
//    myhead.id[0] = 'C';
//    myhead.id[1] = 'L';
    myhead.option = imageinfor();
    myhead.option &= 0xfffffffc;
    myhead.option += (samperpixel & 0x00000003);
    myhead.metrics = imagemetrics();
    myhead.res_unit = -1;
    
    void* pointer;
    pointer = GlobalLock(handle);
    if(pointer == NULL)
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage,"error locking memory .");
        return FALSE;
    }
    
    memcpy(pointer, (void*)&myhead, sizeof(MYHEAD));

    GlobalUnlock(handle);
    return TRUE;
}


BOOL CImg::release(HANDLE handle)
{
    if(GlobalFree(handle)!=NULL)
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage,"memory release error .");
        return FALSE;
    }
    return TRUE;
}


BOOL CImg::resetimagepointer(void)
{
    linenumber = 0;
    dataposition = 0;
    return TRUE;
}

HANDLE CImg::convertToDIB(HANDLE hMyHandle)
{
    HANDLE hOutput;
    LPBYTE pData, pOutput;
    BITMAPINFOHEADER DIBHeader;
    MYHEAD *pMyHeader;
    RGBQUAD RGBPalette;
    DWORD dwLength, dwAddOn;
    DWORD i;

    RGBPalette.rgbReserved = 0;

    pData = (LPBYTE)::GlobalLock(hMyHandle);
    if(pData == NULL)
    {
        return NULL;
    }

/*    if(pData[0] != 'C' || pData[1] != 'L')
    {
        ::GlobalUnlock(hMyHandle);
        return NULL;
    }
*/
    pMyHeader = (MYHEAD*)pData;
    pData += sizeof(MYHEAD);

    DIBHeader.biSize = sizeof(BITMAPINFOHEADER);
    DIBHeader.biWidth = (pMyHeader->metrics) & 0x0000ffff;
    DIBHeader.biHeight = (pMyHeader->metrics) >> 16;
    DIBHeader.biPlanes = 1;
    DIBHeader.biBitCount = (pMyHeader->option >> 2) & 0x003f;
    DIBHeader.biCompression = BI_RGB;
    DIBHeader.biSizeImage = 0;
    switch(pMyHeader->res_unit)
    {
        case 1:     // TIFF
            DIBHeader.biXPelsPerMeter = 300;
            DIBHeader.biYPelsPerMeter = 300;
            break;
        case 2:     // TIFF, DPI
            DIBHeader.biXPelsPerMeter = pMyHeader->h_res0 / pMyHeader->h_res1;
            DIBHeader.biYPelsPerMeter = pMyHeader->v_res0 / pMyHeader->v_res1;
            break;
        case 3:     // TIFF, dots per centimeter
            DIBHeader.biXPelsPerMeter = (long)(pMyHeader->h_res0*2.54/pMyHeader->h_res1+0.5);
            DIBHeader.biYPelsPerMeter = (long)(pMyHeader->v_res0*2.54/pMyHeader->v_res1+0.5);
            break;
        case 4:     // PCX
            DIBHeader.biXPelsPerMeter = 300;
            DIBHeader.biYPelsPerMeter = 300;
            break;
        case 5:     // BMP, dots per meter
            DIBHeader.biXPelsPerMeter = (long)(pMyHeader->h_res0 * 0.0254+0.5);
            DIBHeader.biYPelsPerMeter = (long)(pMyHeader->v_res0 * 0.0254+0.5);
            break;
        default:
            DIBHeader.biXPelsPerMeter = 300;
            DIBHeader.biYPelsPerMeter = 300;
            break;
    }
    DIBHeader.biClrUsed = 0;
    DIBHeader.biClrImportant = 0;

    dwLength = ((DIBHeader.biWidth * DIBHeader.biBitCount + 31) / 32)
        * 4 * (long)DIBHeader.biHeight;
    if(DIBHeader.biBitCount <= 8)
        dwLength += sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) << DIBHeader.biBitCount;
    else
        dwLength += sizeof(BITMAPINFOHEADER);
    hOutput = ::GlobalAlloc(GMEM_MOVEABLE, dwLength);
    if(hOutput == NULL)
    {
        ::GlobalUnlock(hMyHandle);
        return NULL;
    }
    pOutput = (LPBYTE)::GlobalLock(hOutput);
    if(pOutput == NULL)
    {
        ::GlobalUnlock(hMyHandle);
        GlobalFree(hOutput);
        return NULL;
    }
	
	ZeroMemory( pOutput, dwLength );
    memcpy(pOutput, (void*)&DIBHeader, sizeof(BITMAPINFOHEADER));
    pOutput += sizeof(BITMAPINFOHEADER);

    if(DIBHeader.biBitCount == 1)
    {
        if(((pMyHeader->option >> 8) & 0x03) == 0)     // white is zero
        {
            RGBPalette.rgbReserved = 0;
            RGBPalette.rgbBlue = RGBPalette.rgbRed = RGBPalette.rgbGreen = 255;
            memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
            pOutput += sizeof(RGBQUAD);
            RGBPalette.rgbBlue = RGBPalette.rgbRed = RGBPalette.rgbGreen = 0;
            memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
            pOutput += sizeof(RGBQUAD);
        }
        else        // black is zero
        {
            RGBPalette.rgbReserved = 0;
            RGBPalette.rgbBlue = RGBPalette.rgbRed = RGBPalette.rgbGreen = 0;
            memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
            pOutput += sizeof(RGBQUAD);
            RGBPalette.rgbBlue = RGBPalette.rgbRed = RGBPalette.rgbGreen = 255;
            memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
            pOutput += sizeof(RGBQUAD);
        }
    }
    else if(DIBHeader.biBitCount <= 8)
    {
        if((pMyHeader->option & 0x0300) == 0x0300)
        {
            dwLength = (DIBHeader.biWidth * DIBHeader.biBitCount + 7) / 8 *
                (long)DIBHeader.biHeight;
            RGBPalette.rgbReserved = 0;
            for(i=0; i<(DWORD)(1<<DIBHeader.biBitCount); i++)
            {
                RGBPalette.rgbRed = *(pData + dwLength + 3 * i);
                RGBPalette.rgbGreen = *(pData + dwLength + 3 * i + 1);
                RGBPalette.rgbBlue = *(pData + dwLength + 3 * i + 2);
                memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
                pOutput += sizeof(RGBQUAD);
            }
        }
        else if((pMyHeader->option & 0x0300) == 0x0000)
        {
            RGBPalette.rgbReserved = 0;
            for(i=0; i<(DWORD)(1<<DIBHeader.biBitCount); i++)
            {
                RGBPalette.rgbRed = (BYTE)(255 - i * (1 << (8 - DIBHeader.biBitCount)));
                RGBPalette.rgbGreen = (BYTE)(255 - i * (1 << (8 - DIBHeader.biBitCount)));
                RGBPalette.rgbBlue = (BYTE)(255 - i * (1 << (8 - DIBHeader.biBitCount)));
                memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
                pOutput += sizeof(RGBQUAD);
            }
        }
        else
        {
            RGBPalette.rgbReserved = 0;
            for(i=0; i<(DWORD)(1<<DIBHeader.biBitCount); i++)
            {
                RGBPalette.rgbRed = (BYTE)(i * (1 << (8 - DIBHeader.biBitCount)));
                RGBPalette.rgbGreen = (BYTE)(i * (1 << (8 - DIBHeader.biBitCount)));
                RGBPalette.rgbBlue = (BYTE)(i * (1 << (8 - DIBHeader.biBitCount)));
                memcpy(pOutput, (void*)&RGBPalette, sizeof(RGBQUAD));
                pOutput += sizeof(RGBQUAD);
            }
        }
    }

    dwLength = (DIBHeader.biWidth * DIBHeader.biBitCount + 7) / 8;
    dwAddOn = (DIBHeader.biWidth * DIBHeader.biBitCount + 31) / 32 * 4 - dwLength;
    pData += dwLength * (DIBHeader.biHeight - 1);
    if(dwAddOn == 0)
    {
        for(i=0; i<(DWORD)DIBHeader.biHeight; i++)
        {
            memcpy(pOutput, pData, dwLength);
            pOutput += dwLength;
            pData -= dwLength;
        }
    }
    else
    {
        for(i=0; i<(DWORD)DIBHeader.biHeight; i++)
        {
            memcpy(pOutput, pData, dwLength);
            pOutput += dwLength;
            pData -= dwLength;
            memset(pOutput, 0, dwAddOn);
            pOutput += dwAddOn;
        }
    }

    if(DIBHeader.biBitCount == 24)
    {
        BYTE cTemp;
        DWORD j;

        pOutput -= (dwLength + dwAddOn) * DIBHeader.biHeight;
        pData = pOutput + 2;
        for(i=0; i<(DWORD)DIBHeader.biHeight; i++)
        {
            for(j=0; j<(DWORD)DIBHeader.biWidth; j++)
            {
                cTemp = *(pOutput + j * 3);
                *(pOutput + j * 3) = *(pData + j * 3);
                *(pData + j * 3) = cTemp;
            }
            pOutput += dwLength + dwAddOn;
            pData += dwLength + dwAddOn;
        }
    }

    GlobalUnlock(hMyHandle);
    GlobalUnlock(hOutput);

    return hOutput;
}

HANDLE CImg::convertFromDIB(HANDLE hDIB)
{
    if(errorflag)
        return NULL;

    HANDLE hOutput;
    LPBYTE pData, pOutput;
    BITMAPINFOHEADER *pDIBHeader;
    MYHEAD *pMyHeader;
    RGBQUAD *pRGBPalette;
    DWORD dwLength, dwAddOn;
    int i;
    int nDirection;
    BYTE rgbLast[4];

    pData = (LPBYTE)::GlobalLock(hDIB);
    if(pData == NULL)
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage, "Error lock memory.");
        return NULL;
    }

    pDIBHeader = (BITMAPINFOHEADER*)pData;
    pData += sizeof(BITMAPINFOHEADER);

    if(pDIBHeader->biSize != sizeof(BITMAPINFOHEADER))
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage, "Not valid DIB structure.");
        ::GlobalUnlock(hDIB);
        return NULL;
    }
    if(pDIBHeader->biBitCount == 16 || pDIBHeader->biBitCount == 32)
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage, "Bits per pixel not supported.");
        ::GlobalUnlock(hDIB);
        return NULL;
    }

    dwLength = (pDIBHeader->biWidth * pDIBHeader->biBitCount + 7) / 8 *
        abs(pDIBHeader->biHeight) + sizeof(MYHEAD);
    if(pDIBHeader->biBitCount <= 8 && pDIBHeader->biBitCount > 1)
        dwLength += 3 << pDIBHeader->biBitCount;

    hOutput = ::GlobalAlloc(GMEM_MOVEABLE, dwLength);
    pOutput = (LPBYTE)::GlobalLock(hOutput);
    if(pOutput == NULL)
    {
        errorflag = TRUE;
        lstrcpy((LPSTR)errormessage, "Error allocate memory.");
        ::GlobalUnlock(hDIB);
		::GlobalUnlock(hOutput);
        return NULL;
    }

    pMyHeader = (MYHEAD*)pOutput;
    pOutput += sizeof(MYHEAD);

//    pMyHeader->id[0] = 'C';
//    pMyHeader->id[1] = 'L';
    pMyHeader->option = pDIBHeader->biBitCount << 2;
    if(pDIBHeader->biBitCount <= 8)
    {
        if(pDIBHeader->biBitCount == 1)
        {
            float r0,r1;
            r0 = (float)(0.299 * pData[2] + 0.587 * pData[1]
                            + 0.114 * pData[0]);
            r1 = (float)(0.299 * pData[6] + 0.587 * pData[5]
                            + 0.114 * pData[4]);

            if(r0 < r1)
                pMyHeader->option += 1 << 8;
            else
                pMyHeader->option += 0 << 8;
        }
        else
        {
            for(i = 0; i < (1 << pDIBHeader->biBitCount); i++)
            {
                if(pData[0] != pData[1] || pData[0] != pData[2])
                    break;

                if(i == 1)
                {
                    if(pData[0] > rgbLast[0])
                        nDirection = 0;
                    else if(pData[0] < rgbLast[0])
                        nDirection = 1;
                    else
                        break;
                }

                if(i > 1)
                {
                    if((pData[0] > rgbLast[0] && nDirection == 1) ||
                        (pData[0] < rgbLast[0] && nDirection == 0) ||
                        (pData[0] == rgbLast[0]))
                    {
                        break;
                    }
                }

                memcpy(rgbLast, pData, 4);

                pData += sizeof(RGBQUAD);
            }
            pData = (LPBYTE)pDIBHeader + sizeof(BITMAPINFOHEADER);
            if(i != (1 << pDIBHeader->biBitCount))
            {
                pMyHeader->option += 3 << 8;
            }
            else if(pData[0] < 5)
            {
                pMyHeader->option += 1 << 8;
            }
            else
            {
                pMyHeader->option += 0 << 8;
            }
        }
    }
    else
    {
        pMyHeader->option += 2 << 8;
    }
    pMyHeader->metrics = pDIBHeader->biWidth + (abs(pDIBHeader->biHeight) << 16);
    pMyHeader->res_unit = 5;
    pMyHeader->h_res0 = pDIBHeader->biXPelsPerMeter;
    pMyHeader->v_res0 = pDIBHeader->biYPelsPerMeter;

    if(pDIBHeader->biBitCount <= 8)
        pData += sizeof(RGBQUAD) << pDIBHeader->biBitCount;

    dwLength = (pDIBHeader->biWidth * pDIBHeader->biBitCount + 7) / 8;
    dwAddOn = (pDIBHeader->biWidth * pDIBHeader->biBitCount + 31) / 32 * 4 - dwLength;
    if(pDIBHeader->biHeight > 0)
    {
        pData += (dwLength + dwAddOn) * (abs(pDIBHeader->biHeight) - 1);
        for(i=0; i<abs(pDIBHeader->biHeight); i++)
        {
            memcpy(pOutput, pData, dwLength);
            pOutput += dwLength;
            pData -= dwLength + dwAddOn;
        }
    }
    else
    {
        for(i=0; i<abs(pDIBHeader->biHeight); i++)
        {
            memcpy(pOutput, pData, dwLength);
            pOutput += dwLength;
            pData += dwLength + dwAddOn;
        }
    }

    if((pMyHeader->option & 0x0300) == 0x0300)
    {
        pData = (LPBYTE)pDIBHeader + sizeof(BITMAPINFOHEADER);
        pOutput = (LPBYTE)pMyHeader + sizeof(MYHEAD);
        pOutput += dwLength * abs(pDIBHeader->biHeight);
        for(i=0; i<(1<<pDIBHeader->biBitCount); i++)
        {
            pRGBPalette = (RGBQUAD*)pData;
            pOutput[0] = pRGBPalette->rgbRed;
            pOutput[1] = pRGBPalette->rgbGreen;
            pOutput[2] = pRGBPalette->rgbBlue;
            pData += sizeof(RGBQUAD);
            pOutput += 3;
        }
    }

    if(pDIBHeader->biBitCount == 24)
    {
        BYTE cTemp;
        DWORD j;

        pOutput = (LPBYTE)pMyHeader + sizeof(MYHEAD);
        for(i=0; i<abs(pDIBHeader->biHeight); i++)
        {
            for(j=0; j<(DWORD)pDIBHeader->biWidth; j++)
            {
                cTemp = *(pOutput + j * 3);
                *(pOutput + j * 3) = *(pOutput + j * 3 + 2);
                *(pOutput + j * 3 + 2) = cTemp;
            }
            pOutput += dwLength;
        }
    }

    GlobalUnlock(hDIB);
    GlobalUnlock(hOutput);

    return hOutput;
}
