#include "windows.h"
#include "stdio.h"
#include "math.h"
#include "afx.h"
#include "afxwin.h"

#ifndef _CImgExist

#define _CImgExist

typedef struct
{
//    char id[2];
    int		option;
    long	metrics;
    int		res_unit;       // -1     : no_use   
                            // 1,2,3  : for TIFF files
                            // 4      : for PCX files
                            // 5      : for BMP files
    long	h_res0,h_res1;  // not TIFF , h_res0 is the value
                            // otherwise h_res0 is numerator ;
                            // h_res1 is denominator .
    long	v_res0,v_res1;  // see comments above of h_res .
    char	reserved[46];
} MYHEAD;
    
class CImg : public CFile
{
    public :
//  constructor and destructor
        CImg();
        CImg(const char *pszFileName,UINT nOpenFlags);
        ~CImg();
        
//  nOpenFlags :   
//       Sharing and access mode. Specifies the action to take 
//       when opening the file. You can combine options listed below
//       by using the bitwise-OR (|) operator. One access permission
//       and one share option are required; the modeCreate and 
//       modeNoInherit modes are optional. See the Help of CFile
//       constructor for a list of mode options.

    protected :
        int width,height;               // unit : pixel
        int samperpixel;                // samples per pixel. default : 1
        int depth;                      // bits per pixel
        int color;                      // color definition
                                        // 0 : noncolor_whiteiszero
                                        // 1 : noncolor_blackiszero
                                        // 2 : color_RGB
                                        // 3 : color_palette
        unsigned int compression;       // compression method

//  1 : No_Compression 
//  2 : LZW_Compression
//  3 : Group4_compression
//  4 : PackBits_compression
//  5 : BI_RLE8_compression
//  6 : BI_RLE4_compression
//  7 : PCX_compression
//  8 : CCITT_1D_compression
//  9 : Group3_compression
        
        HANDLE memoryhandle;            // image data (decompressed)
        int filehandle;                 // CFile::m_hFile
        UINT openflags;                 // open options
        char filextension[4];           // MS-DOS file extension
        BOOL valid;                     // if file extension name is valid
                                        // TRUE : valid
        long dataposition;              // used by function readline()
        int linenumber;                 // used by function readline()

    public :
    
        char imagefilename[180];         // filename with path
        BOOL errorflag;                 // TRUE indicates error occurence
        char errormessage[100];         // error description
        
        
    public :
        virtual int imageinfor();                                 
//  return information of image
//  int CImg::imageinfor()
//  return value :
//  bit     0 : if this image file exists and be opened (1 : existent)
//  bit     1 : whether format is correct (by extension name.)
//  bit  2--7 : bits per pixel
//  bit  8--9 : color definition
//  bit 10-15 : compression method

        virtual long imagemetrics(); 
//  return width and height of image . Low word : x . High word : y .       
        virtual BOOL openimagefile(char *filename,UINT nOpenFlags);              
//  open file when use default constructor      
        virtual void closeimagefile(); 
//  close file when use default constructor     
        virtual BOOL release(HANDLE handle);
//  release memory handle by GlobalFree
        virtual HANDLE readimage(void)=0;
        virtual HANDLE readimage(int x,int y,int width,int height)=0;
//  read data from imagefile . filepointer will be reset .      
        virtual BOOL writeimage(long metrics,int option,
                                        HANDLE handle)=0;
//  write image in memory to disk . filepointer will be reset
//  metrics : height << 16 + width
//  handle  : memory handle of image data
//  option  : bit   0--1  : samples per pixel . 0 presents 4 samples
//            bit   2--7  : bits per pixel
//            bit   8--9  : color definition
//            bit  10--15 : compression method

        virtual BOOL readline(unsigned char* buffer,int *index)=0;
//  this functions above support scanline_based_io and need file-
//  pointer points to right position . so they can't be interrupted by
//  any functions which will change the filepointer ( such as 
//  readimage() and writeimage() ) .
        BOOL resetimagepointer(void);
//  reset linenumber for another read
        BOOL isvalidfile(void);
//  if file extension is valid.
        virtual BOOL writemyhead(HANDLE handle);
//  write head for decompressed data in memory .
        HANDLE convertToDIB(HANDLE hMyHandle);
//  convert my format to DIB format.
        HANDLE convertFromDIB(HANDLE hDIB);
//  convert DIB to my format.
};

#endif
