#ifndef _CTifExist

#include "class_ba.h" 

#define _CTifExist

#define MAXNUMBER ((unsigned long)0xffffffffL)
#define NoOuterDataFile
// if NoOuterDataFile defined, no outer data file will be readed when initializing.

#define MultiImageWriting
// define this to support multi-image-writing. (if write to a already-exist tiff
//      file, the data is written at end of file and original data not destroyed.)

#ifndef _Group4
#define _Group4
#endif

#ifdef _Group4
#ifndef _Group4_Do

#define _Group4_Do

#define White                   0
#define Black                   1

#define Largest_code            2560

#define Pixels_per_byte         8  
#define Bits_per_byte           8  
#define Last_bit_in_a_byte      7
#define Last_bit_mask           1

#define Invalid                -1 
#define Extra_positions         25

#define VL3                    -3
#define VL2                    -2
#define VL1                    -1
#define V0                      0
#define VR1                     1
#define VR2                     2
#define VR3                     3
#define P                       4
#define H                       5
#define EOFB                    6

typedef struct
{
    int pixel[9]; 
} byte_descriptor;

typedef struct
{
    int value;
    int child_zero;
    int child_one;
} node;

#endif
#endif

class CTiff : public CImg
{
    public :
        CTiff();
        CTiff(const char *pszFileName,UINT nOpenFlags,
                   int image_no = 1,LPSTR szPath = NULL);
//  image_no : which image in tiff file will be processed. (1_based)
        ~CTiff();
        
    public :
    
        char palette[768];
        int resolutionunit;     // 1 : dot per ?
                                // 2 : dot per inch
                                // 3 : dot per cm
        long h_res0,h_res1;     // resolution value .
        long v_res0,v_res1;     // ?_res0 : numerator ;
                                // ?_res1 : denominator .
        int photometric;        // describe color definition in image

    protected :

        int flag;               // flag = 1 : II , flag = 0 : MM
        BOOL IFDNumber;         // TRUE : IFD = 1    FALSE : IFD > 1
                                // IFD > 1 means more than one image .
        int bitpersample;       // bits per sample. default : 1
                                // multi-bitpersample not supported
        int planar;             // indicate (RGBRGB) or (RRGGBB)
                                // default : 1
        long ColorMapPos;       // position of colormap in file
        unsigned long rowsperstrip; 
                                // rows per strip. default : 2^32 - 1
        long StripOffsetsPos;   // position of strip offsets in file
        int numofstrip;         // number of strip . if 1 , strip offset
                                // just be StripOffsetsPos
        int typeofstripoff;     // strip offset int or long
                                // 3 : int      4 : long
        long StripBytesPos;     // position of strip data length in file
        int typeofstripbytes;   // strip data length int or long
                                // 3 : int      4 : long
        BOOL istiled;           // if it is a tiled image
        int tilewidth;          // width for each tile
        int tilelength;     // length for each tile
        int tilexnum;           // number of tile in width;
        int tileynum;           // number of tile in height;
        int numoftile;          // number of tile .
        long TileOffsetsPos;    // position of tile offsets in file
        long TileBytesPos;      // position of tile data length in file
        int typeoftilebytes;    // tile data length int or long
                                // 3 : int      4 : long
        int Orientation;        // only support 1--4.  5--8 is not supported.
                                // only support in readimage(void).
        long XResolutionPos;    // position of x-resolution in file
        long YResolutionPos;    // position of y-resolution in file
        long newsubfiletype;    // not used in this program module .
        char cLookupTable[256]; // for FillOrder = 2
        short nPredictor;       // Differencing Predictor.

    protected :
        
//        int bit_offset;                 // used by writeimage() and
//        int code_size;                  // LZW_write_code();
//        unsigned char code_buffer[260]; // for LZW_compression;
        
    public :

        virtual BOOL openimagefile(char *filename,UINT nOpenFlags,
                       int image_no = 1,LPSTR szPath = NULL);
//  open file when use default constructor      
        virtual HANDLE readimage(void);
        virtual HANDLE readimage(int x,int y,int width,int height);
//  read data from imagefile . filepointer will be reset .      
        virtual BOOL writeimage(long metrics,int option,HANDLE handle);
//  write image in memory to disk . filepointer will be reset .                                       
        virtual BOOL readline(unsigned char* buffer,int *index);
//  this functions above support scanline_based_io and need file-
//  pointer points to right position . so they can't be interrupted by
//  any functions which will change the filepointer ( such as 
//  readimage() and writeimage() ) .
        virtual BOOL writemyhead(HANDLE handle);

//  read data from file
        unsigned int getword(void);
        unsigned long getlong(void);

    protected :
        virtual BOOL setdata(int image_no,LPSTR szPath);      
//  set private data .
        BOOL getimage(unsigned char* pointer,DWORD fileoffset,
                    DWORD inbytes,int x,int y,long *usedbyte);
//  read part of image , used by readimage() .
        BOOL readPackBitsline(unsigned char* pointer,int x);
        BOOL writePackBitsline(unsigned char* pointer,int x);
//  read or write one line which is compressed with PackBits_compression .
        void writetag(int name,int tag,DWORD length,DWORD offset);
//  write tiff tag into file .
//        void write_code(int code);
//  used by writeimage() , for LZW_compression;
        BOOL Read_LZW_Data(unsigned char* pInput, unsigned char* pOutput, int iWidth);
//  used to read LZW_compression data.


#ifdef _Group4

    protected :
    
        void setstaticdata(void);
        BOOL write_bits_c(char* string_ptr,long size);
        BOOL write_run_length(int length, int color,long size);
        HANDLE grp4comp(HANDLE hindata,long inbytes,int width,
            int height,long *outbytes);
        
        int  find_run_length_code(UINT color);
        int  read_bit(BOOL reset = FALSE);
        void write_bits_d(UINT length, UINT color);
        HANDLE grp4decomp(HANDLE hindata,DWORD inbytes,
            int width,int height);
        HANDLE grp3decomp(HANDLE hindata,DWORD inbytes,
            int width,int height);

    protected :

        static char *largest_colorless_code;
        static unsigned char  write_one[Pixels_per_byte];
        static unsigned char  write_zero[Pixels_per_byte];
        
        HANDLE static_data_handle;
        char* white_terminating_code;    // 09 * 64
        char* black_terminating_code;    // 13 * 64
        char* white_make_up_code;        // 13 * 40
        char* black_make_up_code;        // 14 * 40
        byte_descriptor* table;          // sizeof(int) * 9 * 256
        node* white_tree;                // sizeof(int) * 3 * 208
        node* black_tree;                // sizeof(int) * 3 * 208

    protected :
        
        int  bit_place_mark;
        long byte_place_mark;
        int  comp_write_init_flag;
        unsigned char *output_area;
        
        int decomp_write_init_flag;
        int decomp_read_init_flag;
        unsigned char *input_area;
        
        long l_v_f_r_l;
        long T4Options;
        char szLibraryPath[144];
        
        int  FillOrder;

#endif

    protected :
        int bit_offset;                 // used by writeimage() and
        int code_size;                  // write_code();
        unsigned char code_buffer[260]; // for LZW compression.
        void write_code(int code);      // Used for LZW compression.

};


#endif
