/////////////////////////////////////////////////////////////////////////
// File Name:		DirLine.h
// File Function:	1. Interface for CDirLine class
//					2. Define some inline methods to speedup the program
//
//				Developed by: Yefeng Zheng
//			(c)copyright to Tsinghua THOCR Lab, 2001
//					All Right Reserved
//////////////////////////////////////////////////////////////////////////

#ifndef __DIRLINE_H__
#define __DIRLINE_H__

#include "ConnTree.h"
#include "ImageObj.h"
#include "math.h"

class CDirLine : public CObject
{
DECLARE_SERIAL(CDirLine)
public :
	int		m_nCharWidth;	//Character width estimated
	int		m_nCharHeight;  //Character height estimeated
    BOOL	m_bIsHorLine;   //Horizontal or Veritical line

	int		CharWidth[128];//Character width histogram, used to estimate character width
	int		CharHeight[128];//Character height histogram, used to estimate character height

//Variables used to split a image into several strips to speed up the algorithm
	int		m_nStrip; //Strip number of the image devided
	int		m_nCurTree;//Current strip under analysis
    CConnTree** m_pTree;//Each strip analysis by a CConnTree instance

public :
    CDirLine() ;
    ~CDirLine() ;
    int		FreeMem(); //Free memory

///////////////////////////////////////////////////////////////////////////////////
//	Methods to create Directional Single Connected Chain
///////////////////////////////////////////////////////////////////////////////////
    //Extract runlengths
	int		BuildConnTree ( CImage& Img, RECT rRange ) ;
    //Get pixels on a horizontal image row
	int		AquireHorLineData ( BYTE *p, int w, int h, int n, int start, int end, int *buf, int ImgType, int Channel=0, BOOL bUpSideDown = TRUE ) ;
    //Get pixels on a vertical image column
	int		AquireVerLineData ( BYTE *p, int w, int h, int n, int start, int end, int *buf, int ImgType, int Channel=0, BOOL bUpSideDown = TRUE ) ;
    //Extract column runlength
	int		GetColumnRunLength ( BYTE *p, int w, int h, int column, int start, int end, LineValley *valley, BOOL bUpSideDown ) ;
    //Extract row runlength
	int		GetRowRunLength ( BYTE *p, int w, int h, int row, int start, int end, LineValley *valley, BOOL bUpSideDown ) ;
    //Extract runlength on gray image
	int		ValleyDetect ( int *p, int start, int end, LineValley *valley, int *s, int Depth, int MaxLen ) ;
	//Column runlength smoothing
	int		ColRunLenSmooth(BYTE *p, int w, int h, int col, LineValley *valley, int& Valleys);
	//Row runlength smoothing
	int		RowRunLenSmooth(BYTE *p, int w, int h, int row, LineValley *valley, int& Valleys);	


///////////////////////////////////////////////////////////////////////////////////
//	Others
///////////////////////////////////////////////////////////////////////////////////
	int		GetAverCharWidth(BOOL bLastStrip);
	//Extract all connected components
	int		EraseConnComp(CImage &Image, int nMinWidth, int nMaxWidth, int nMinHeight, int nMaxHeight);

};

#endif //__DIRLINE_H__