////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		PolyLine.cpp
// File Function:	Read and write polyline information
//
//				Developed by: Yefeng Zheng
//			   First created: Dec. 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PolyLine.h"

/*****************************************************************************
/*	Name:		ReadPolyLine
/*	Function:	Read polylines from a file
/*	Parameter:	fnInput   -- File name
/*				PolyLine  -- Polylines
/*				nPolyLine -- Number of polylines	
/*	Return:		0 -- Succeed
/*****************************************************************************/
int ReadPolyLine(char* fnInput, POLYLINE *&PolyLine, int &nPolyLine)
{
	FILE	*fp = fopen(fnInput, "rt");
	if( fp == NULL)			return	-1;
	char	buf[1000];
	fgets( buf, 1000, fp );
	sscanf( buf, "Line=%d", &nPolyLine );
	if( nPolyLine<0 || nPolyLine >10000) return -1;

	PolyLine = (POLYLINE*)malloc( sizeof( POLYLINE )*nPolyLine );
	if( PolyLine == NULL )	return -1;

	for( int i=0; i<nPolyLine; i++ )
	{
		if( fgets( buf, 1000, fp ) == NULL )
		{
			AfxMessageBox("Read polyline results failed!" );
			return -1;
		}
		sscanf( buf, "Point=%d", &PolyLine[i].nPnt );
		if( PolyLine[i].nPnt >= MAXPOLYPOINT )	return -1;
		char *p = buf;
		for( int j=0; j<PolyLine[i].nPnt; j++ )
		{
			p = strchr( p+1, '(');
			if( p== NULL )	return -1;
			sscanf( p, "(%d,%d)", &PolyLine[i].Pnt[j].x, &PolyLine[i].Pnt[j].y );
		}
	}
	fclose( fp );
	return 0;
}

/*****************************************************************************
/*	Name:		WritePolyLine
/*	Function:	Write polylines to a file
/*	Parameter:	fnOuput   -- Output file name
/*				PolyLine  -- Polylines
/*				nPolyLine -- Number of polylines	
/*	Return:		0 -- Succeed
/*****************************************************************************/
int	WritePolyLine( char *fnOutput, POLYLINE *PolyLine, int nPolyLine)
{
	FILE *fp = fopen( fnOutput, "wt" );
	fprintf( fp, "Line=%3d\n", nPolyLine );
	for( int i=0; i<nPolyLine; i++ )
	{
		fprintf( fp, "Point=%3d  ", PolyLine[i].nPnt );
		for( int j=0; j<PolyLine[i].nPnt; j++ )
			fprintf( fp, "(%4d,%4d)  ", PolyLine[i].Pnt[j].x, PolyLine[i].Pnt[j].y);
		fprintf( fp, "\n" );
	}
	fclose( fp );
	return 0;
}