////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		ReadHMM.cpp
// File Function:	Read HMM model from file.
//
//				Developed by: Yefeng Zheng
//			   First created: Sept. 2003
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <math.h>
#include "RuleLineAPI.h"

#define	MAXLINE			1000	
char	buf[MAXLINE];				//buf to read file

/****************************************************************************************************
/*	Name:		ReadHMM
/*	Function:	Read HMM model with discrete observation
/*	Parameter:	fnHMM -- File of HMM model
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int ReadHMM( char *fnHMM, HMM_MODEL &model )
{
	FILE *fpHMM = fopen( fnHMM, "rt" );
	if( fpHMM == NULL )
	{
		printf( "Can not open %s !\n", fnHMM );
		return -1;
	}
	int		i, tmp;
	double	nTotal = 0;

	fgets( buf, MAXLINE, fpHMM );//read out "Text peak type:"
	nTotal = 0;
	for( i=0; i<MAXPEAK; i++ )
	{
		fgets( buf, MAXLINE, fpHMM );
		sscanf( buf, "%d:%lf", &tmp, &model.TextPeakProb[i] );
		nTotal += model.TextPeakProb[i];
	}

	//The parameters are already optimized, no normalization is neeeded.
//Normalize
//	for( i=0; i<MAXPEAK; i++ )
//		model.TextPeakProb[i] /= nTotal;

	fgets( buf, MAXLINE, fpHMM );//Read out "Line peak type:"
	nTotal = 0;
	for( i=0; i<MAXPEAK; i++ )
	{
		fgets( buf, MAXLINE, fpHMM );
		sscanf( buf, "%d:%lf", &tmp, &model.LinePeakProb[i] );
		nTotal += model.LinePeakProb[i];
	}
//Normallize
//	for( i=0; i<MAXPEAK; i++ )
//		model.LinePeakProb[i] /= nTotal;

	fgets( buf, MAXLINE, fpHMM );//Read out "Line gap:"
	fgets( buf, MAXLINE, fpHMM );
	sscanf( buf, "Min:%d	Max:%d", &model.nMinLineGap, &model.nMaxLineGap );
	if( model.nMaxLineGap - model.nMinLineGap <= 0 || model.nMaxLineGap - model.nMinLineGap >= MAXLINEGAP )
	{
		printf( "Line gap distribution error!\n" );
		return -1;
	}
	nTotal = 0;
	for( i=0; i<=model.nMaxLineGap-model.nMinLineGap; i++ )
	{
		fgets( buf, MAXLINE, fpHMM );
		sscanf( buf, "%d  %lf", &tmp, &model.LineGapDist[i] );
		nTotal += model.LineGapDist[i];
	}
	for( i=0; i<=model.nMaxLineGap-model.nMinLineGap; i++ )
		model.LineGapDist[i] /= nTotal;
	fclose( fpHMM );

//Scaling
	for( i=0; i<MAXPEAK; i++ )
	{
		model.TextPeakProb[i] = log( model.TextPeakProb[i] );
		model.LinePeakProb[i] = log( model.LinePeakProb[i] );
	}
	for( i=0; i<=model.nMaxLineGap-model.nMinLineGap; i++ )
	{
		if( model.LineGapDist[i] < 1e-5 )
			model.LineGapDist[i] = log( 1e-5 );
		else
			model.LineGapDist[i] = log( model.LineGapDist[i] );
	}
	return 0;
}