////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		TrainHMM.cpp
// File Function:	Train HMM model
//
//				Developed by: Yefeng Zheng
//			   First created: Nov. 2003
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RuleLineAPI.h"
#include "Tools.h"
#include "TrainHMMDiscrete.h"

CRLTrainHMM::CRLTrainHMM()
{
	strcpy( m_fnTrainHMMConfig, "" );
	return;
}


CRLTrainHMM::~CRLTrainHMM()
{
	return;
}

/****************************************************************************************************
/*	Name:		RLSetTrainConfig
/*	Function:	Set configure file of the training set
/*	Parameter:	fnTrainConfig -- Configure file of the training set
/*	Return:		0  -- Succeed
/***************************************************************************************************/
int CRLTrainHMM::RLSetTrainConfig( char *fnTrainConfig )
{
	strcpy( m_fnTrainHMMConfig, fnTrainConfig );
	return 0;
}

/****************************************************************************************************
/*	Name:		RLTrainRuleLineModel
/*	Function:	Train HMM model
/*	Parameter:	NULL
/*	Return:		0  -- Succeed
/***************************************************************************************************/
int CRLTrainHMM::RLTrainRuleLineModel( )
{
	if( strcmp( m_fnTrainHMMConfig, "" ) == 0 )
	{
		printf( "Please set configure for training first!\n" );
		return -1;
	}
	for( int i=0; i<MAXPEAK; i++ )
	{
		m_HMM_Model.LinePeakProb[i] = 0;
		m_HMM_Model.TextPeakProb[i] = 0;
	}
	for( i=0; i<MAXLINEGAP; i++ )
		m_HMM_Model.LineGapDist[i] = 0;
	m_HMM_Model.nMaxLineGap = 0;
	m_HMM_Model.nMinLineGap = 0;
	TrainHMM( m_fnTrainHMMConfig, TRUE, m_HMM_Model );
	return 0;
}

/****************************************************************************************************
/*	Name:		RLOutputRuleLineModel
/*	Function:	Output trained HMM model
/*	Parameter:	NULL
/*	Return:		0  -- Succeed
/***************************************************************************************************/
int CRLTrainHMM::RLOutputRuleLineModel( char *fnHMM_Model )
{
	OutputHMM( fnHMM_Model, m_HMM_Model );
	return 0;
}