/////////////////////////////////////////////////////////////////////////
// File NAME:		TestAPI.cpp
// File Function:	Test the following APIs: TextIDAPI, RuleLineAPI, ZoneSegAPI.
//
//				Developed by:Yefeng Zheng
//					All Right Reserved
/////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TestAPI.h"
#include "TextIDAPI.h"
#include "RuleLineAPI.h"
#include "ZoneSegAPI.h"
#include "ArabicTextIDAPI.h"
#include "Tools.h"

CWinApp theApp;

/****************************************************************************************************
/*	Name:		TestArabicLayerSeparate
/*	Function:	Test layer separate
/*	Parameter:	fnImg			-- Input image file
/*				fnTreeLineShort -- Trained classification tree for short text lines
/*				fnTreeLineLong  -- Trained classification tree for long text lines
/*				fnTreeWordSize  -- Trained classification tree for word level classification
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestArabicLayerSeparate( char *fnImg, char *fnTreeLineShort, char *fnTreeLineLong, char *fnTreeWordSize )
{
	printf( "Processing %s\n", fnImg );

	CArabicLayerSeparate	LayerSeparate;

	if( LayerSeparate.TISetImage( fnImg ) != NULL )
		return -1;

	if( LayerSeparate.TIWordSegmentation( ) != NULL )
		return -1;

	if( LayerSeparate.TISetClassifier( fnTreeLineShort, fnTreeLineLong, fnTreeWordSize ) != 0 )
		return -1;
	if( LayerSeparate.TIClassification() != 0 )
		return -1;

	char fnZone[MAX_PATH];
	ChangeFileExt( fnZone, fnImg, "zone" );
	if( LayerSeparate.TIOutputZone( fnZone ) != 0 )
		return -1;

//	WORD_ZONE *pWord = NULL;
//	int nWord; 
//	if( LayerSeparate.TIGetWord( pWord, nWord ) != 0 )	//You do not need to free pWord. It is managed by the API.
//		return -1;

/*	char fnLayer[MAX_PATH];
	sprintf( fnLayer, "C:\\%s_P.tif", GetFileNameWithoutExt( fnImg ) );
	if( LayerSeparate.TISaveLayer( fnLayer, 1 ) != 0 )
		return -1;
	sprintf( fnLayer, "C:\\%s_H.tif", GetFileNameWithoutExt( fnImg ) );
	if( LayerSeparate.TISaveLayer( fnLayer, 2 ) != 0 )
		return -1;
	sprintf( fnLayer, "C:\\%s_N.tif", GetFileNameWithoutExt( fnImg ) );
	if( LayerSeparate.TISaveLayer( fnLayer, 3 ) != 0 )
		return -1;
*/
	return 0;
}

/****************************************************************************************************
/*	Name:		TestLayerSeparate
/*	Function:	Test layer separate
/*	Parameter:	fnImg        -- Input image file
/*				fnClassifier -- Trained fisher classifier
/*				fnMRF		 -- MRF model for post-processing
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestLayerSeparate( char *fnImg, char *fnClassifier, char *fnMRF )
{
	printf( "Processing %s\n", fnImg );
	char fnZone[MAX_PATH];

	CTILayerSeparate	LayerSeparate;

	if( LayerSeparate.TISetImage( fnImg ) != NULL )
		return -1;

	if( LayerSeparate.TIWordSegmentation( ) != NULL )
		return -1;

	if( LayerSeparate.TISetClassifier( fnClassifier ) != 0 )
		return -1;
	if( LayerSeparate.TIClassification() != 0 )
		return -1;

	if( LayerSeparate.TISetMRFModel( fnMRF ) != 0 )
		return -1;
	if( LayerSeparate.TIMRFPostProcessing( ) != 0 )
		return -1;

	ChangeFileExt( fnZone, fnImg, "zone" );
	if( LayerSeparate.TIOutputZone( fnZone ) != 0 )
		return -1;

	char fnLayer[MAX_PATH];
	sprintf( fnLayer, "P_%s.tif", GetFileNameWithoutExt( fnImg ) );
	if( LayerSeparate.TISaveLayer( fnLayer, 1 ) != 0 )
		return -1;
	sprintf( fnLayer, "H_%s.tif", GetFileNameWithoutExt( fnImg ) );
	if( LayerSeparate.TISaveLayer( fnLayer, 2 ) != 0 )
		return -1;
	sprintf( fnLayer, "N_%s.tif", GetFileNameWithoutExt( fnImg ) );
	if( LayerSeparate.TISaveLayer( fnLayer, 3 ) != 0 )
		return -1;
	return 0;
}

/****************************************************************************************************
/*	Name:		TestTrainMRFModel
/*	Function:	Test MRF model training module
/*	Parameter:	fnTrainConfig -- Configure file of the training set
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestTrainMRFModel( char *fnTrainConfig)
{
	CTITrainMRFModel TrainMRFModel;

	if( TrainMRFModel.TISetTrainConfig( fnTrainConfig ) != 0 )
		return -1;
	if( TrainMRFModel.TITrainMRFModel( ) != 0 )
		return -1;
	if( TrainMRFModel.TIOutputMRFModel( "C:\\MRF.txt" ) != 0 )
		return -1;
	return 0;
}

/****************************************************************************************************
/*	Name:		TestTrainClassifier
/*	Function:	Test classifier training module
/*	Parameter:	fnTrainConfig -- Configure file of the training set
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestTrainClassifier( char *fnTrainConfig)
{
	CTITrainClassifier TrainClassifier;
	if( TrainClassifier.TISetTrainConfig( fnTrainConfig ) != 0 )
		return -1;
	if( TrainClassifier.TITrainClassifier( ) != 0 )
		return -1;
	if( TrainClassifier.TIOutputClassifier( "C:\\Test.Fisher" ) != 0 )
		return -1;
	return 0;
}

/****************************************************************************************************
/*	Name:		TestRuleLineDetection
/*	Function:	Test rule line detection
/*	Parameter:	fnImg		-- Input image
/*				fnHMM_Model -- HMM model for rule line detection
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestRuleLineDetection( char *fnImg, char *fnHMM_Model )
{
	CRLLineDetection	LineDetect;
	LineDetect.RLSetImage( fnImg );
	LineDetect.RLSetHMMModel( fnHMM_Model );

	if( LineDetect.RLLineDetect( ) != 0 )
	{
		printf( "Rule line detection failed!\n" );
		return -1;
	}

	LINE *pLine;
	int nLine;
	LineDetect.RLGetLines( pLine, nLine );

	char fnLine[MAX_PATH];
// Old line file format
/*	ChangeFileExt( fnLine, fnImg, "line" );
	FILE *fpLine = fopen( fnLine, "wt" );
	if( fpLine == NULL )	return -1;
	fprintf( fpLine, "Total Lines=%d\n", nLine );
	for( int i=0; i<nLine; i++ )
		fprintf( fpLine, "(%5d %5d)\t(%5d %5d)\n", pLine[i].StPnt.x, pLine[i].StPnt.y, 
				pLine[i].EdPnt.x, pLine[i].EdPnt.y );
	fclose( fpLine );
*/

// New line file format
	ChangeFileExt( fnLine, fnImg, "fdr" );
	FILE *fpLine = fopen( fnLine, "wb" );
	if( fpLine == NULL)			return	-1;
	fwrite( &nLine, sizeof(int), 1, fpLine);
	for(int i=0; i<nLine; i++)
	{	
		fwrite( &pLine[i].StPnt.x, sizeof(int), 1, fpLine);
		fwrite( &pLine[i].StPnt.y, sizeof(int), 1, fpLine);
		fwrite( &pLine[i].EdPnt.x, sizeof(int), 1, fpLine);
		fwrite( &pLine[i].EdPnt.y, sizeof(int), 1, fpLine);
		fwrite( &pLine[i].nType,   sizeof(int), 1, fpLine);
	}
	fclose( fpLine );
	free( pLine );

	bool	bRuleLine = LineDetect.RLLineExist();

/*	FILE *fp = fopen( "C:\\RuleLineJudge.txt", "at" );
	fprintf( fp, "%s\t%d\n", fnImg, bRuleLine );
	fclose( fp );
*/

	if(	bRuleLine )	// Automatic judge if rule lines exist.
	{
		LineDetect.RLLineRemove();
		BYTE *pCleanImg;
		int w;
		int h;
		LineDetect.RLGetCleanImage( pCleanImg, w, h);
		free( pCleanImg );

		char fnCleanImg[MAX_PATH];
		sprintf( fnCleanImg, "..\\%s", fnImg );
		LineDetect.RLSaveCleanImage( fnCleanImg );
	}
	return 0;
}

/****************************************************************************************************
/*	Name:		TestRuleLineTrainHMM
/*	Function:	Test the HMM model training module
/*	Parameter:	fnTrainConfig -- Configure file of the training set
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestRuleLineTrainHMM( char *fnTrainConfig )
{
	CRLTrainHMM	TrainHMM;
	TrainHMM.RLSetTrainConfig( fnTrainConfig );
	TrainHMM.RLTrainRuleLineModel(  );
	TrainHMM.RLOutputRuleLineModel( "TestHMM.hmm" );
	return 0;
}

/****************************************************************************************************
/*	Name:		TestZoneSeg
/*	Function:	Test the zone segmentation API
/*	Parameter:	fnImg -- Input image file
/*				nMode -- Different algorithms. 1 -- XYCut, 2 -- Docstrum, 3 -- Voronoi
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int TestZoneSeg( char *fnImg, int nMode )
{
	char	fnZone[MAX_PATH];
	CZSZoneSeg ZoneSeg;
	printf( "Processing %s image ...\n", fnImg );
	ZoneSeg.ZSSetImage( fnImg );
	switch( nMode )
	{
	case 1:
		ZoneSeg.ZSXycut( NULL );
		break;
	case 2:
		ZoneSeg.ZSDocstrum( NULL );
		break;
	case 3:
		ZoneSeg.ZSVoronoi( NULL );
		break;
	default:
		return -1;
	}
	ChangeFileExt( fnZone, fnImg, "zone" );
	ZoneSeg.ZSOutputZone( fnZone );
	return 0;
}

/****************************************************************************************************
/*	Name:		_tmain
/*	Function:	main function
/*	Parameter:	
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/***************************************************************************************************/
int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{ 
//	TestTrainMRFModel( "C:\\Work\\Laser\\API\\models\\MRF.train" );
//	TestTrainClassifier( "C:\\Work\\Laser\\API\\models\\Classifier.train" );
	TestLayerSeparate( argv[1], 
		"C:\\Work\\Distribution\\UMD_API\\models\\Classifier.clf", 
		"C:\\Work\\Distribution\\UMD_API\\models\\MRF.mrf" );

//	TestRuleLineDetection( argv[1], argv[2]);
//	TestRuleLineTrainHMM( "..\\models\\RuleLineHMM.train");

//	TestZoneSeg( argv[1], 3 );

/*	if( argc == 2 )
		TestArabicLayerSeparate( argv[1], "TextLine_Short.tree", "TextLine_Long.tree", "Word_Size.tree" );
	else if( argc == 5 )
		TestArabicLayerSeparate( argv[1], argv[2], argv[3], argv[4] );
	else
		return -1;
*/	return 0;
}
