////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		ExtractFeature.cpp
// File Function:	Extract features
//
//				Developed by: Yefeng Zheng
//			   First created: Jan, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <math.h>
#include "imageobj.h"
#include "imagetool.h"
#include "ExtractFeature.h"
#include "ExtractGaborFeature.h"
#include "Tools.h"
#include "img.h"

int		nAverWidth=20, nAverHeight=20;	//Normalization factor of character width and height
BYTE	ImgByte[512*512];				//Buffer to convert bit image to byte image

/**************************************************************************************************
/*	Name:		ExtractFeature
/*	Function:	Extract feature for one image
/*	Parameter:	ImgZone     -- Zone block image
/*				Feature     -- Extracted features
/*	Return:		0  -- Succeed
/*************************************************************************************************/
int	ExtractFeature( CImage &ImgZone, double *Feature )
{	
	int		nFeature = 0, i;
	double	tmpFeature[256];

	int  nImgWb = ImgZone.GetLineSizeInBytes();
	int	 nImgWidth = ImgZone.GetWidth();
	int  nImgHeight = ImgZone.GetHeight();
	BYTE *pImgZone = ImgZone.LockRawImg();
				
	CRect BoundBox;
	if( nImgWidth * nImgHeight >= 512*512 )	return -1;
	int nBlackPixels = BitToByte( pImgZone, nImgWb, nImgWidth, nImgHeight, ImgByte, nImgWidth, nImgHeight, BoundBox );
	
	ExtractWHFeature( ImgZone, tmpFeature );
	for( i=0; i<4; i++ )
		Feature[nFeature++] = tmpFeature[i];
	//Extract contour features
	ExtractContourFeature( ImgByte, nImgWidth, nImgHeight, tmpFeature );
	for( i=0; i<2; i++ )
		Feature[nFeature++] = tmpFeature[i];

	ExtractProjFeature( ImgZone, tmpFeature );
	for( i=0; i<2; i++ )
		Feature[nFeature++] = tmpFeature[i];

	int  nConnComp;
	int	 *nConnPixels;
	RECT *pConnComp;
	CImageTool::ExtractConnComp( pImgZone, nImgWb*8, nImgHeight, pConnComp, nConnPixels, nConnComp, TRUE, 8, FALSE );
	ExtractCCFeature( pConnComp, nConnPixels, nConnComp, tmpFeature );
	free( pConnComp );
	free( nConnPixels );
	for( i=0; i<10; i++ )
		Feature[nFeature++] = tmpFeature[i];

	int nChgDimension = 5;
	double HChgFeature[5], VChgFeature[5];
	if( ExtractStrokeComplexityFeature( ImgByte, nImgWidth, nImgHeight, BoundBox, HChgFeature, VChgFeature, nChgDimension ) != 0 )
		return -1;
	for( i=0; i<nChgDimension; i++ )
	{
		Feature[nFeature++] = HChgFeature[i];
		Feature[nFeature++] = VChgFeature[i];
	}

	int nRLDimension = 5;
	double HFeature[5], VFeature[5], LT_RB_Feature[5], RT_LB_Feature[5];
	if( ExtractStraightnessFeature( ImgByte, nImgWidth, nImgHeight, BoundBox, HFeature, VFeature, nRLDimension ) != 0 )
		return -1;
	if( ExtractSlantStraightnessFeature( ImgByte, nImgWidth, nImgHeight, BoundBox, LT_RB_Feature, RT_LB_Feature, nRLDimension ) != 0 )
		return -1;
	for( i=0; i<nRLDimension; i++ )
	{
		Feature[nFeature++] = HFeature[i];	
		Feature[nFeature++] = VFeature[i];
		Feature[nFeature++] = LT_RB_Feature[i];
		Feature[nFeature++] = RT_LB_Feature[i];
	}

	//Extract bi-level co-occurrence features
	BWIMG img;
	img.xsize = nImgWidth;
	img.ysize = nImgHeight;
	img.xres  = 300;
	img.yres  = 300;
	img.rowsize = (nImgWidth+7)/8;
	img.data = (BYTE*)malloc( sizeof(int)*img.rowsize*img.ysize );
	for( i=0; i<nImgHeight; i++ )
		memcpy( img.data+i*img.rowsize, pImgZone+(nImgHeight-1-i)*nImgWb, img.rowsize );

	/* Set last null bits to 0 */
	unsigned char xendnullbits, xendbytemask;
	xendnullbits = (img.rowsize<<3) - img.xsize;
	xendbytemask = 255 << xendnullbits;

	for (int y = 0; y < img.ysize; y ++)
		*(img.data + img.rowsize * (y+1) -1 ) &= xendbytemask;
	ExtractTextureFeature( img, tmpFeature );
	for( i=0; i<92; i++ )
		Feature[nFeature++] = tmpFeature[i];
	
	ImgZone.UnlockRawImg( );

/*	ExtractGaborFeature( ImgByte, nImgWidth, nImgHeight, tmpFeature );
	for( i=0; i<16; i++ )
		Feature[nFeature++] = tmpFeature[i];
*/
	free( img.data );
	return 0;
}

/**************************************************************************************************
/*	Name:		ExtractFeature
/*	Function:	Extract feature for one image
/*	Parameter:	ImgZone     -- Zone block image
/*				WordSeg     -- Detailed word segmentation information
/*				nIndex		-- Index to the WORD ZONE in WordSeg
/*				Feature     -- Extracted features
/*	Return:		0  -- Succeed
/*************************************************************************************************/
int	ExtractFeature( CImage &ImgZone, CWordSegment &WordSeg, int nIndex, double *Feature )
{	
	int		nFeature = 0, i;
	double	tmpFeature[256];

	int  nImgWb = ImgZone.GetLineSizeInBytes();
	int	 nImgWidth = ImgZone.GetWidth();
	int  nImgHeight = ImgZone.GetHeight();
	BYTE *pImgZone = ImgZone.LockRawImg();
				
	ZONE	Word = WordSeg.m_pWord[nIndex];
	CRect BoundBox;
	if( Word.w * Word.h >= 512*512 )	return -1;
	int nBlackPixels = BitToByte( pImgZone, nImgWb, nImgWidth, nImgHeight, ImgByte, nImgWidth, nImgHeight, BoundBox );
	
	// Extract features based to the block size
	ExtractWHFeature( ImgZone, tmpFeature );
	for( i=0; i<4; i++ )
		Feature[nFeature++] = tmpFeature[i];
	// Extract contour features
	// Warning: These features are not scale invariant
	ExtractContourFeature( ImgByte, nImgWidth, nImgHeight, tmpFeature );
	for( i=0; i<2; i++ )
		Feature[nFeature++] = tmpFeature[i];

	// Extract features of the variance of projection profiles
	ExtractProjFeature( ImgZone, tmpFeature );
	for( i=0; i<2; i++ )
		Feature[nFeature++] = tmpFeature[i];

	int  nConnComp = min( 256, Word.child_count);
	int	 nConnPixels[256];
	RECT pConnComp[256];
	for( i=0; i<nConnComp; i++ )
	{
		ZONE	CC = WordSeg.m_pComp[Word.child[i]];
		pConnComp[i].left	= CC.x;
		pConnComp[i].top	= CC.y;
		pConnComp[i].right	= CC.x + CC.w - 1;
		pConnComp[i].bottom = CC.y + CC.h - 1;
		nConnPixels[i] = CC.nBlackPixels;
	}
	// Extract features based on CC
	ExtractCCFeature( pConnComp, nConnPixels, nConnComp, tmpFeature );
	for( i=0; i<10; i++ )
		Feature[nFeature++] = tmpFeature[i];

	int nChgDimension = 5;
	double HChgFeature[5], VChgFeature[5];
	// Extract crossing-count histogram features
	if( ExtractStrokeComplexityFeature( ImgByte, nImgWidth, nImgHeight, BoundBox, HChgFeature, VChgFeature, nChgDimension ) != 0 )
		return -1;
	for( i=0; i<nChgDimension; i++ )
	{
		Feature[nFeature++] = HChgFeature[i];
		Feature[nFeature++] = VChgFeature[i];
	}

	int nRLDimension = 5;
	double HFeature[5], VFeature[5], LT_RB_Feature[5], RT_LB_Feature[5];
	// Extract horizontal and vertical run-length histogram features
	if( ExtractStraightnessFeature( ImgByte, nImgWidth, nImgHeight, BoundBox, HFeature, VFeature, nRLDimension ) != 0 )
		return -1;
	// Extract run-length histogram features on the major and minor diagonal directions.
	// Warning: These features are not exactly scale invariant
	if( ExtractSlantStraightnessFeature( ImgByte, nImgWidth, nImgHeight, BoundBox, LT_RB_Feature, RT_LB_Feature, nRLDimension ) != 0 )
		return -1;
	for( i=0; i<nRLDimension; i++ )
	{
		Feature[nFeature++] = HFeature[i];	
		Feature[nFeature++] = VFeature[i];
		Feature[nFeature++] = LT_RB_Feature[i];
		Feature[nFeature++] = RT_LB_Feature[i];
	}

	//Extract bi-level co-occurrence features
	BWIMG img;
	img.xsize = nImgWidth;
	img.ysize = nImgHeight;
	img.xres  = 300;
	img.yres  = 300;
	img.rowsize = (nImgWidth+7)/8;
	img.data = (BYTE*)malloc( sizeof(int)*img.rowsize*img.ysize );
	for( i=0; i<nImgHeight; i++ )
		memcpy( img.data+i*img.rowsize, pImgZone+(nImgHeight-1-i)*nImgWb, img.rowsize );

	/* Set last null bits to 0 */
	unsigned char xendnullbits, xendbytemask;
	xendnullbits = (img.rowsize<<3) - img.xsize;
	xendbytemask = 255 << xendnullbits;

	for (int y = 0; y < img.ysize; y ++)
		*(img.data + img.rowsize * (y+1) -1 ) &= xendbytemask;

	// Extract texture features.
	// Warning: These features are not scale invariant.
	ExtractTextureFeature( img, tmpFeature );
	for( i=0; i<92; i++ )
		Feature[nFeature++] = tmpFeature[i];
	
	ImgZone.UnlockRawImg( );

/*	ExtractGaborFeature( ImgByte, nImgWidth, nImgHeight, tmpFeature );
	for( i=0; i<16; i++ )
		Feature[nFeature++] = tmpFeature[i];
*/
	free( img.data );
	return 0;
}

/*****************************************************************************
/*	Name:		ExtractContourFeature
/*	Function:	Extrac contour pixel ratio features. Two features: contour pixel
/*				ratio to region area and contour pixel ratio to black pixels
/*	Parameter:	ImgByte    -- Image, one byte for one pixel
/*				nImgWidth  -- Image width
/*				nImgHeight -- Image height
/*				fpFeature  -- Feature file pointer
/*	Return:		0 -- Succeed
/*****************************************************************************/
int	ExtractContourFeature( BYTE *ImgByte, int nImgWidth, int nImgHeight, double *Feature )
{
	int i, j;
	int nContour=0, nBlackPixel=0;
	for( i=0; i<nImgHeight; i++ )
	{
		for( j=0; j<nImgWidth; j++ )
		{
			if( ImgByte[i*nImgWidth+j] == 1 )
			{
				nBlackPixel++;
				if( i== 0 || i== nImgHeight-1 || j==0 || j==nImgWidth-1 )
					nContour++;				
				else 
				{
					if( ImgByte[(i-1)*nImgWidth+j] == 0 )		nContour++;
					else if( ImgByte[(i+1)*nImgWidth+j] == 0 )	nContour++;
					else if( ImgByte[ i*nImgWidth+j-1] == 0 )	nContour++;
					else if( ImgByte[ i*nImgWidth+j+1] == 0 )	nContour++;
				}
			}
		}
	}
	Feature[0] = 1.0*nContour/nImgWidth/nImgHeight;
	Feature[1] = 1.0*nContour/nBlackPixel;
	return 0;
}

/*****************************************************************************
/*	Name:		ExtractWHFeature
/*	Function:	Extrac width/height ratio feature
/*	Parameter:	Img		   -- Image
/*				fpFeature  -- Feature file pointer
/*	Return:		0 -- Succeed
/*****************************************************************************/
int	ExtractWHFeature( CImage &Img, double *Feature )
{		
	int nImgWidth = Img.GetWidth();
	int nImgHeight = Img.GetHeight();
	double	nBlackRatio = Img.GetBlackPixelRatio( );
	Feature[0] = 1.0*nImgWidth/nAverWidth;
	Feature[1] = 1.0*nImgHeight/nAverHeight;
	Feature[2] = 1.0*nImgWidth/nImgHeight;
	Feature[3] = nBlackRatio;
	return 0;
}


/************************************************************************************
/*	Name:		ExtractCCFeature
/*	Function:	Extrac features of connect components. Including following features
/*				- Small CC number
/*				- Total CC number
/*				- Small CC ratio
/*				- Mean of CC width
/*				- Variance of CC width
/*				- Mean of CC height
/*				- Variance of CC height
/*				- Mean of CC area
/*				- Varince of CC area
/*				- Overlap rate of bounding boxes of CC
/*	Parameter:	pConnComp   -- Connect components
/*				nConnPixels -- Number of black pixels for each connect components
/*				nConnComp   -- Number of connect components
/*				fpFeature   -- Feature file pointer
/*	Return:		0 -- Succeed
/************************************************************************************/
int ExtractCCFeature( RECT *pConnComp, int *nConnPixels, int nConnComp, double *Feature )
{
	int i=0, j;
	int nSmallCC = 0;
	int nTotalConnComp = nConnComp;

	while( i<nConnComp )
	{//Filter small connected components
		int w = pConnComp[i].right - pConnComp[i].left + 1;
		int h = pConnComp[i].bottom - pConnComp[i].top + 1;
		if( w<=5 && h<=5 )
		{
			nSmallCC++;
			for( j=i; j<nConnComp-1; j++ )
			{
				pConnComp[j] = pConnComp[j+1];
				nConnPixels[j] = nConnPixels[j+1];
			}
			nConnComp--;
		}
		else
			i++;
	}
	double nSmallCCRatio = 1.0*nSmallCC/nTotalConnComp;

	double nAverCCWidth=0, nAverCCHeight=0, nArea=0, nAreaOverlap = 0;
	double nVarCCWidth=0, nVarCCHeight=0, nVarArea=0;

	if( nConnComp != 0 )
	{
		for(i=0; i<nConnComp; i++ )
		{
			int w = pConnComp[i].right - pConnComp[i].left + 1;
			int h = pConnComp[i].bottom - pConnComp[i].top + 1;
			nAverCCWidth += w;
			nAverCCHeight += h;
			nArea += w*h;
			for( j=i+1; j<nConnComp; j++ )
			{
				int nMaxLeft = (int)max( pConnComp[i].left, pConnComp[j].left );
				int nMinRight = (int)min( pConnComp[i].right, pConnComp[j].right );
				int nMaxTop = (int)max( pConnComp[i].top, pConnComp[j].top );
				int nMinBottom = (int)min( pConnComp[i].bottom, pConnComp[j].bottom );
				if( nMaxLeft < nMinRight && nMaxTop < nMinBottom )
					nAreaOverlap += (nMinRight-nMaxLeft) * (nMinBottom-nMaxTop);
			}
		}
		nAverCCWidth /= nConnComp;
		nAverCCHeight /= nConnComp;
		nAreaOverlap = 2*nAreaOverlap/nArea;
		nArea /= nConnComp;
		
		for(i=0; i<nConnComp; i++ )
		{
			int w = pConnComp[i].right - pConnComp[i].left + 1;
			int h = pConnComp[i].bottom - pConnComp[i].top + 1;
			nVarCCWidth += (w-nAverCCWidth)*(w-nAverCCWidth);
			nVarCCHeight += (h-nAverCCHeight)*(h-nAverCCHeight);
			nVarArea += (w*h-nArea)*(w*h-nArea);
		}
		nVarCCWidth = sqrt( nVarCCWidth/nConnComp );
		nVarCCHeight = sqrt( nVarCCHeight/nConnComp );
		nVarArea = sqrt( nVarArea/nConnComp );
	}

	Feature[0] = 1.0*nSmallCC;
	Feature[1] = 1.0*nTotalConnComp;
	Feature[2] = nSmallCCRatio;
	Feature[3] = nAverCCWidth/nAverWidth;
	Feature[4] = nVarCCWidth/nAverWidth;
	Feature[5] = nAverCCHeight/nAverHeight;
	Feature[6] = nVarCCHeight/nAverHeight;
	Feature[7] = nArea/(nAverWidth*nAverHeight);
	Feature[8] = nVarArea/(nAverWidth*nAverHeight);
	Feature[9] = nAreaOverlap;
	return 0;
}

/*****************************************************************************
/*	Name:		ExtractProjFeature
/*	Function:	Extrac varince of horizontal and vertical project profile
/*	Parameter:	Img		   -- Image
/*				fpFeature  -- Feature file pointer
/*	Return:		0 -- Succeed
/*****************************************************************************/
int ExtractProjFeature( CImage &Img, double *Feature )
{
	int w = Img.GetWidth();
	int h = Img.GetHeight();
	double	*Proj;
	int		nProj;
	CImageTool::DoHorProject(Img, Proj, nProj, CRect(0, 0, 0, 0 ) );
	double *HProj = (double*)malloc( sizeof(double)*nProj );
	for( int i=0; i<nProj; i++ )
		HProj[i] = 1.0*Proj[i]/w;

	double MeanHProj, VarHProj;
	Mean( HProj, nProj, MeanHProj );
	Var( HProj, nProj, VarHProj, MeanHProj );
	Feature[0] = VarHProj;
	free( Proj );
	free( HProj );

	CImageTool::DoVerProject( Img, Proj, nProj, CRect(0, 0, 0, 0 ) );
	double *VProj = (double*)malloc( sizeof(double)*nProj );
	for( i=0; i<nProj; i++ )
		VProj[i] = 1.0*Proj[i]/h;
	double MeanVProj, VarVProj;
	Mean( VProj, nProj, MeanVProj );
	Var( VProj, nProj, VarVProj, MeanVProj );
	Feature[1] = VarVProj;
	free( Proj );
	free( VProj );

	return 0;
}
