/////////////////////////////////////////////////////////////////
// File Name:		ImageTool.h
// File Function:	Interface for CImageTool class
//
//				Developed by: Yefeng Zheng
//					All Right Reserved
//////////////////////////////////////////////////////////////////////
#ifndef __IMAGE_TOOL_H__
#define	__IMAGE_TOOL_H__

#include "ImageObj.h"

class CImageTool : public CObject  
{
public:
	CImageTool();
	virtual ~CImageTool();
//////////////////////////////////////////////////////////////////////////////////
//	Methods for copying Image
//////////////////////////////////////////////////////////////////////////////////
	//Copy an image block 
	static int CopyBlock ( BYTE *ps, int xs, int ys, int wInBytesS, BYTE *pd, int xd, int yd, int wInBytesD, int w, int h, int Type, BOOL bUpSideDown );
	//Copy image with more parameters for adjusting
	static int CopyImage ( CImage &ImgS, int xs, int ys, CImage &ImgD, int xd, int yd, int w, int h);
	//Copy image
	static int CopyImage ( CImage &ImgS, int xs, int ys, int w, int h, CImage &ImgD);
	// Copy binary block with byte allign to speed up copy image speed
	static int CopyBinaryImageByteAllign ( CImage &ImgS, int xs, int ys, int w, int h, CImage &ImgD );
	
//////////////////////////////////////////////////////////////////////////////////
//	Methods for cutting Image
//////////////////////////////////////////////////////////////////////////////////
	//Brush the top part of a block to white
	static int CutImageTop ( BYTE *pImg, int wb, int h, POINT Left, POINT Right, int Type );
	//Brush the bottom part of a block to white
	static int CutImageBottom ( BYTE *pImg, int wb, int h, POINT Left, POINT Right, int Type );
	//Brush the left part of a block to white
	static int CutImageLeft ( BYTE *pImg, int wb, int h, POINT Top, POINT Bottom, int Type );
	//Brush the right part of a block to white
	static int CutImageRight ( BYTE *pImg, int wb, int h, POINT Top, POINT Bottom, int Type );
	//Cut a block image out of the whole image
	static int CutImage ( CImage &ImgS, POINT TopL, POINT TopR, POINT BtmL, POINT BtmR, CImage &ImgD );

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Methods for eraseing pixels
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//Set a vertical line to the indicated gray level
	static int SetVerLineToGrayscale ( BYTE *p, int WidthInBytes, int Height, int x, int y, int h, BYTE Gray, BOOL bUpSideDown );
	//Set a horizontal line to the indicated gray level
	static int SetHorLineToGrayscale ( BYTE *p, int WidthInBytes, int Height, int x, int y, int w, BYTE Gray, BOOL bUpSideDown );
	//Set a vertical line to the indicated color
	static int SetVerLineToColor ( BYTE *p, int WidthInBytes, int Height, int x, int y, int h, COLORREF Color, BOOL bUpSideDown );
	//Set a horizontal line to the indicated color
	static int SetHorLineToColor ( BYTE *p, int WidthInBytes, int Height, int x, int y, int w, COLORREF Color, BOOL bUpSideDown );
	//Set a vertical line to black
	static int SetVerLineToBlack ( BYTE *p, int WidthInBytes, int Height, int x, int y, int h, BOOL bUpSideDown );
	//Set a vertical line to white
	static int SetVerLineToWhite ( BYTE *p, int WidthInBytes, int Height, int x, int y, int h, BOOL bUpSideDown );
	//Set a horizontal line to black
	static int SetHorLineToBlack ( BYTE *p, int WidthInBytes, int Height, int x, int y, int w, BOOL bUpSideDown );
	//Set a horizontal line to white. Her p point to the whole image
	static int SetHorLineToWhite ( BYTE *p, int WidthInBytes, int Height, int x, int y, int w, BOOL bUpSideDown );
	//Set a horizontal line to white. Here p point to the image line
	static int SetHorLineToWhite ( BYTE *p, int x, int y, int w );
	//Set a line to specific value for color/gray/binary images
	static int SetLine ( BYTE *p, int WidthInBytes, int Height, int x, int y, int Len, BOOL bIsHorLine, int ImageType, BYTE Gray, BOOL bUpSideDown );
	//Draw a line on an image, used for debug
	static BOOL DrawLine ( CImage &Img, POINT s, POINT e );
	//Draw a rectangle on an image, used for debug
	static BOOL DrawRect ( CImage &Img, RECT rc );
	//Earse black strip of a binary image
	static int EraseBlackStrip( CImage &Img );
	//Earse black strip of a binary image, and save the processed result
	static int EraseBlackStrip ( char* sFnImage );
	//Dilate image used for line detection on edge images	//Yefeng Zheng 02/23/2003
	static int Dialate ( CImage &Img, CImage &DImg );

/////////////////////////////////////////////////////////////////////////////////////
//	Methods for testing white/black 
/////////////////////////////////////////////////////////////////////////////////////	
	//Test if the pixels on a horizontal line are all white
	static BOOL AllVerWhite ( BYTE *p, int wb, int h, POINT s, POINT e);
	//Test if the pixels on a vertial line are all white
	static BOOL AllHorWhite ( BYTE *p, int wb, int h, int x, int y, int w );
	//Test if the pixels on a horizontal/vertical line are all white
	static BOOL AllWhite ( BYTE *p, int wb, int h, POINT s, POINT e );
	//Test if the pixels on a line are all white in a gray level image
	static BOOL GrayAllWhite ( BYTE *p, int wb, int h, POINT s, POINT e, BYTE Gray );
	//Test if the pixels on a line are all white in a color image
	static BOOL ColorAllWhite ( BYTE *p, int wb, int h, POINT s, POINT e, int Channel, BYTE Gray );

	
/////////////////////////////////////////////////////////////////////////////////////////////////////
//	Skew correction
/////////////////////////////////////////////////////////////////////////////////////////////////////
	//Copy pixels, written in assemble language to accelate, called by BlockCopyBi
	static int BlockCopyBi0 ( BYTE *ImgS, int xs, int ys, int wInBytesS, BYTE *ImgD, int xd, int yd, int wInBytesD, int w, int h );
	//Copy pixels, written in assemble language to accelate, used for skew correction of binary images
	static int BlockCopyBi ( BYTE *ImgS, int xs, int ys, int wInBytesS, BYTE *ImgD, int xd, int yd, int wInBytesD, int w, int h );
	//Skew correction for binary image
	static CImage * BinaryImageSkewCorrection ( CImage* _SrcImg, double Angle );
	//Skew correction for binary image, another interface
	static int	BinaryImageSkewCorrection ( CImage &_SrcImg, CImage &_DstImg, double Angle );
	//Skew correction then cut extra border
	static int	SkewCorrectionCut( CImage &Img, double dSkewAngle, CImage &CorrectedImg );
	//Skew correction then cut extra border
	static int	SkewCorrectionCut( CImage &Img, double dSkewAngle );
	//Skew correction for color image 
	static CImage *ColorImageSkewCorrection ( CImage* _SrcImg, double Angle );
	//Rotate binary image, called by Zoom()
	static int	Rotate ( BYTE *ImgS, int SrcXs, int SrcXe, int SrcWdInBytes, int h, BYTE *ImgD, int DstXs, int DstWdInBytes );
	//Upside down
	static int UpsideDown( BYTE *ImgS, int w, int h, BYTE *&ImgD );
/////////////////////////////////////////////////////////////////////////////////////////////////////
//	Other tools
/////////////////////////////////////////////////////////////////////////////////////////////////////
	//Shift image
	static int	ShiftImage(BYTE* pByteS, int w, int h, int wb, BYTE* pByte, int RowShift, int ColShift);
	//Get the bounding box of a character image, processing all image types
	//Extend char or cell boundary box to include all black pixels as possible
	static int	GetCharBound(CImage &Image, POINT &TopL, POINT &TopR, POINT &BtmL, POINT &BtmR);
	//Get the bounding box of a image, only process binary image
	static int	GetBound(CImage &Image, RECT &rRange);
	//Zoom image in Y direction
	static void	ZoomY ( BYTE *ImgS,  BYTE *ImgD, int h, int nh, int wInBytes );
	//Zoom image
	static int	Zoom ( BYTE *ImgS, BYTE *ImgD, int wInBytes, int h, int nwInBytes, int nh );
	//Get black pixel ratio in an image block
	static double GetBlackPixelRatio ( LPBYTE FAR *lpLine,  RECT rcBound );
	static double GetBlackPixelRatio ( CImage &Img,  RECT rcBound );
	//Get the number of black pixels in an image block
	static int	GetBlackPixelNum ( BYTE *p, int wb, int hh, int left, int top, int w, int h ) ;
	//Calculate the number of black pixels on a line
	static int	PixelsOnLine(CImage&  Image, int xb, int yb, int xe, int ye);
	//Do horizontal projection
	static int	DoHorProject(CImage &Img, double *&Proj, int &nProj, CRect rc = CRect(0, 0, 0, 0) );
	//Do skewed horizontal projection
	static int DoSkewHorProject( CImage &Img, double nSkewAngle, double *&Proj, int &nProj  );
	//Do vertical projection
	static int	DoVerProject(CImage &Img, double *&Proj, int &nProj, CRect rc = CRect(0, 0, 0, 0) );
	//Count black pixels on a line
	static int PixelsOnHorLine(CImage&  Image, int xb, int yb, int xe, int ye);
	//Smooth projection
	static int	SmoothProject(double *Proj, int nProj);
	//Dump projection to an image file for debugging
	static int	DumpProject(char* snFileName, double *prj, int nProj, BOOL bHor=TRUE, int nMaxProj=0);

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Extract connected component
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//Extract connected component
	static int	ExtractConnComp( BYTE *Line, int w, int h, RECT *&pConnComp, int *&nConnPixels, int &nConnComp, BOOL bBlackConn, int Connectness, BOOL bByte );
	//Extract row runlength, 1 pixels per bit
	static int	GetRowRunLengthBit( BYTE *Line, int w, int h, int wb, int row, POINT *RunLength, int &nRunLength, BOOL bBlackConn );
	//Extract row runlength, 1 pixels per byte
	static int	GetRowRunLength( BYTE *Line, int w, int h, int row, POINT *RunLength, int &nRunLength, BOOL bBlackConn );

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Bit Byte conversion
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	static int	BitToByte( BYTE* BitImg, int wb, int w, int h, BYTE *&ByteImg, int w2, int h2, CRect &BoundBox );
	static int	BitToByte( BYTE* BitImg, int wb, int w, int h, BYTE *&ByteImg);
	static int	ByteToBit( BYTE* pByteImg, int w, int h, BYTE *RawByte, int wb=0, BYTE BlackPixel=0);
};

#endif // __IMAGE_TOOL_H__
