////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		TestCorrectness.cpp
// File Function:	Test if a pixel is assigned to one and only one layer
//
//				Developed by: Yefeng Zheng
//			   First created: April 2005
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TextIDAPI.h"
#include "ImageObj.h"

extern CImage	m_Img;			//Image class

/********************************************************************************************
/*	Name:		TestExclusion
/*	Function:	Test if two images have the same black pixel.
/*				We can use this function to test that if a pixel is assigned to only one layer
/*	Parameter:	p1	-- Data for image 1
/*				p2	-- Data for image 2
/*				w	-- Width of the image
/*				h	-- Height of the image
/*	Return:		0	-- No block pixel is shared by two images.
/*				-1	-- Two images share the same black pixel.
/********************************************************************************************/
int TestExclusion( BYTE *p1, BYTE *p2, int w, int h )
{
	int wb = w/8;
	for( int i=0; i<h; i++ )
	{
		for( int j=0; j<wb; j++ )
		{
			if( (*p1 & *p2) != 0 )
				return -1;
			p1++;
			p2++;
		}
	}
	return 0;
}

// Test if all pixels are assigned to a layer, and only one layer
/********************************************************************************************
/*	Name:		TITestCorrectness
/*	Function:	Test if a pixel is assigned to one and only one layer
/*	Parameter:	NULL
/*	Return:		0	-- Succeed
/********************************************************************************************/
int CTILayerSeparate::TITestCorrectness( )
{
	// Get images of all layers
	BYTE *pImgP = NULL;
	int wP, hP;
	TIGetLayer( pImgP, wP, hP, TYPE_PRINT_TEXT );
	BYTE *pImgH = NULL;
	int wH, hH;
	TIGetLayer( pImgH, wH, hH, TYPE_HANDWRITING );
	BYTE *pImgN = NULL;
	int wN, hN;
	TIGetLayer( pImgN, wN, hN, TYPE_NOISE );
	
	// Test image sizes
	if( wP != wH || wP != wN || wH != wN )
		return -1;
	if( hP != hP || hP != hN || hH != hN )
		return -1;

	// Test if a pixels belongs to only one layer
	int nExclusion = TestExclusion( pImgP, pImgH, wP, hP );
	if( nExclusion != 0 )	return -1;
	nExclusion = TestExclusion( pImgP, pImgN, wP, hP );
	if( nExclusion != 0 )	return -1;
	nExclusion = TestExclusion( pImgH, pImgN, wP, hP );
	if( nExclusion != 0 )	return -1;

// Test if all pixels are assigned to a layer
	// Get the merged image of all layers	
	int	nSizeByte	= wP/8*hP;
	BYTE *pImgMerge = (BYTE*)malloc( nSizeByte );
	BYTE *pP = pImgP;
	BYTE *pH = pImgH;
	BYTE *pN = pImgN;
	BYTE *pM = pImgMerge;
	for( int i=0; i<nSizeByte; i++ )
	{
		*pM = *pP | *pH | *pN;
		pP++;	pH++;	pN++;
		pM++;
	}

	// Test if the merged image is the same as the original image
	BYTE *pImg = m_Img.LockRawImg( );
	int w = m_Img.GetWidth();
	int h = m_Img.GetHeight();
	int wb = m_Img.GetLineSizeInBytes();
	if( w != wP || h != hP )
		return -1;

	pM = pImgMerge;
	BYTE	*pI;
	int	nResult = 0;
	for( i=0; i<h; i++ )
	{
		pI = pImg+(h-1-i)*wb;
		for( int j=0; j<wb; j++ )
		{
			if( *pI != *pM )
			{
				nResult = -1;
				break;
			}
			pM++;
			pI++;
		}
		if( nResult )	break;
	}
	m_Img.UnlockRawImg();

	delete pImgMerge;
	delete pImgP;
	delete pImgH;
	delete pImgN;

	return nResult;
}
