////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		TrainClassifier.cpp
// File Function:	API for classifier training
//
//				Developed by: Yefeng Zheng
//			   First created: Nov. 2003
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TextIDAPI.h"
#include "Classify.h"
#include "Tools.h"
#include "ImageObj.h"
#include "ImageTool.h"
#include "Output.h"
#include "ExtractFeature.h"
#include "FisherTrain.h"

CTITrainClassifier::CTITrainClassifier()
{ 
	strcpy( m_fnTrainClassifierConfig, "" );
	m_Classifier.Eff = NULL;
	m_Classifier.nMean1 = NULL;
	m_Classifier.nVar1 = NULL;
	m_Classifier.nMean2 = NULL;
	m_Classifier.nVar2 = NULL;
	m_Classifier.nClassLabel1 = NULL;
	m_Classifier.nClassLabel2 = NULL;
	m_Classifier.nTH = NULL;
	m_Classifier.nClassLabel = NULL;
	return; 
}

CTITrainClassifier::~CTITrainClassifier()
{
	FreeFisherModel( m_Classifier );
	return ;
}

/******************************************************************************
/*	Name:		TISetTrainList
/*	Function:	Set configure file for the training set
/*	Parameter:	fnTrainConfig -- Configure file for the training set
/*	Return:		0  -- Succeed
/*****************************************************************************/
int	CTITrainClassifier::TISetTrainConfig( char *fnTrainConfig )
{
	strcpy( m_fnTrainClassifierConfig, fnTrainConfig );
	return 0;
}

/******************************************************************************
/*	Name:		TITrainClassifier
/*	Function:	Train classifier
/*	Parameter:	NULL
/*	Return:		0  -- Succeed
/*****************************************************************************/
int CTITrainClassifier::TITrainClassifier( )
{
	int		i, j;
	double	***Feature;
	TRAIN_CONFIG	TrainConfig;	

	ParseTrainConfig( m_fnTrainClassifierConfig, TrainConfig );
	GetSampleNum( m_fnTrainClassifierConfig, TrainConfig );

	TrainConfig.ClassLabel[0] = TYPE_PRINT_TEXT;
	TrainConfig.ClassLabel[1] = TYPE_HANDWRITING;		
	TrainConfig.ClassLabel[2] = TYPE_NOISE;
	TrainConfig.nTotalClass = 0;
	for( i=0; i<TYPE_TOTAL_CLASS; i++ )
	{
		if( TrainConfig.bTrain[i] )
			TrainConfig.nTotalClass++;
		else
		{
			for( int j=i; j<TYPE_TOTAL_CLASS-1; j++ )
			{
				TrainConfig.NumClass[j] = TrainConfig.NumClass[j+1];
				TrainConfig.ClassLabel[j] = TrainConfig.ClassLabel[j+1];
			}
		}
	}
	double	nMin = 10000;
	for( i=0; i<TrainConfig.nTotalClass; i++ )
		nMin = min( nMin, TrainConfig.NumClass[i] );
	for( i=0; i<TrainConfig.nTotalClass; i++ )
		TrainConfig.nRatio[i] = nMin/TrainConfig.NumClass[i];

//Allocate memory
	for( i=0; i<TrainConfig.nTotalClass; i++ )
		TrainConfig.NumClass[i] = (int)(1.5*nMin);
	Feature = (double***)malloc(sizeof(double**)*TrainConfig.nTotalClass );
	if( Feature == NULL )		return -1;
	for( i=0; i<TrainConfig.nTotalClass; i++ )
	{
		Feature[i] = (double**)malloc(sizeof(double*)*TrainConfig.NumClass[i]);
		if( Feature[i] == NULL )
			return -1;
		for( j=0; j<TrainConfig.NumClass[i]; j++ )
		{
			Feature[i][j] = (double*)malloc(sizeof(double)*FEATURE_DIMENSION);
			if( Feature[i][j] == NULL )
				return -1;
			ZeroMemory( Feature[i][j], sizeof(double)*FEATURE_DIMENSION);
		}
	}
	ExtractTrainFeature( m_fnTrainClassifierConfig, TrainConfig, Feature );
	AllocateClassifier( m_Classifier, TrainConfig.nTotalClass, FEATURE_DIMENSION );
	TrainFisher( Feature, TrainConfig.NumClass, m_Classifier );
	
	for( i=0; i<TrainConfig.nTotalClass; i++ )
	{
		for( j=0; j<(int)(1.5*nMin); j++ )
			free( Feature[i][j] );
		free( Feature[i] );
	}
	free( Feature );

	return 0;
}

/******************************************************************************
/*	Name:		TIOutputClassifier
/*	Function:	Output trained classifier
/*	Parameter:	fnClassifier  -- File name for the trained classifier
/*	Return:		0  -- Succeed
/*****************************************************************************/
int CTITrainClassifier::TIOutputClassifier( char *fnClassifier )
{
	FILE	*fpClassifier = fopen( fnClassifier, "wt" );
	if( fpClassifier == NULL )
	{
		printf( " Can not open %s to write!\n", fnClassifier );
		exit(1);
	}
	fprintf( fpClassifier, "Type=4 Class=%d\n", m_Classifier.nClassNum );

	int		nClassifier = 0;
	for(int i=0; i<m_Classifier.nClassNum-1; i++ )
	{
		for(int j=i+1; j<m_Classifier.nClassNum; j++ )
		{
			fprintf( fpClassifier, "Fisher Efficient:\n" );
			for( int m=0; m<m_Classifier.nDimension; m++ )
				fprintf( fpClassifier, "%e  ", m_Classifier.Eff[nClassifier][m] );
			fprintf( fpClassifier, "\n" );
			fprintf( fpClassifier, "TH=%f\n", m_Classifier.nTH[nClassifier] );
			fprintf( fpClassifier, "Class1=%d Mean1=%e  Delta1=%e\n", 
				m_Classifier.nClassLabel1[nClassifier], m_Classifier.nMean1[nClassifier], m_Classifier.nVar1[nClassifier]);
			fprintf( fpClassifier, "Class2=%d Mean2=%e  Delta2=%e\n", 
				m_Classifier.nClassLabel2[nClassifier], m_Classifier.nMean2[nClassifier], m_Classifier.nVar2[nClassifier]);
			nClassifier++;
		}
	}
	fclose( fpClassifier );
	return 0;
}

