////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		Bit2Byte.cpp
// File Function:	Bit to Byte and Byte to Bit image format conversion
//
//				Developed by: Yefeng Zheng
//			   First created: Jan, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

/******************************************************************************
/*	Name:		BitToByte
/*	Function:	Convert 1 bit/pixel to 1 byte/pixel
/*				Reserved for DIB 4 byte allignment
/*	Parameter:	RawByte -- Raw image
/*				w -- Real image width
/*				h -- Image height
/*				Line -- Converted image
/*				BoundBox -- Boundbox of char image
/*				width -- Image width, used to DIB 4 byte allignment
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/*****************************************************************************/
int	BitToByte( BYTE* RawByte, int wb, int w, int h, BYTE *Line, int w2, int h2, CRect &BoundBox )
{
	if( h2 < h )	return -1;
	BoundBox.left = w;		BoundBox.right = 0;
	BoundBox.top = h;		BoundBox.bottom = 0;
	int i, j, k, TotalPixels = 0;
	BYTE	TempRawByte;
	ZeroMemory( Line, w2*h2*sizeof(BYTE) );
	for(i=0; i<h; i++)
	{
		for( j=0; j<wb; j++)
		{
			TempRawByte = RawByte[i*wb+j];
			k=7; 
			while( k>=0 )
			{
				if( TempRawByte >= 1<<k && j*8+7-k < w)
				{
					Line[(h-1-i)*w2+j*8+7-k] = 1;
					TempRawByte -= 1<<k;
					TotalPixels ++;
					BoundBox.left	= min( BoundBox.left, j*8+7-k );
					BoundBox.right	= max( BoundBox.right, j*8+7-k );
					BoundBox.top	= min( BoundBox.top, h-1-i );
					BoundBox.bottom  = max( BoundBox.bottom, h-1-i );
				}
				k--;
			}
		}
	}
	return TotalPixels;
}

/******************************************************************************
/*	Name:		ByteToBit
/*	Function:	Convert 1 byte/pixel to 1 bit/pixel
/*	Parameter:	Line -- 1 byte/pixel image
/*				w -- Image width
/*				h -- Image height
/*				RawByte -- Converted 1 bit/pixel image
/*	Return:		0  -- Succeed
/*				-1 -- Failed
/*****************************************************************************/
int	ByteToBit( BYTE* Line, int w, int h, BYTE *RawByte)
{
	int		Bytes = (w+7)/8, i, j, k;
	ZeroMemory( RawByte, Bytes*h*sizeof(BYTE) );
	for(i=0; i<h; i++)
	{
		for( j=0; j<Bytes; j++)
		{
			for( k=8*j; k<min( 8*j+8, w ); k++ )
				if( Line[i*w+k] == 1)
					RawByte[i*Bytes+j] += 1<<(7-k%8);
		}
	}
	return 0;
}
