/////////////////////////////////////////////////////////////////////////
// File Name:		DirLineDSCC.cpp
// File Function:	Build Directional Single Connected Chain
//
//				Developed by: Zheng Yefeng
//			(c)copyright to Tsinghua THOCR Lab, 2001
//					All Right Reserved
//////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DirLine.h"

IMPLEMENT_SERIAL(CDirLine, CObject, 0)

CDirLine::CDirLine()
{		
}

CDirLine::~CDirLine()
{
}

/***************************************************************************
/*	Name:		BuildConnTree
/*	Function:	Extract runlengths 
/*	Parameter:	Img -- Form image
/*				rRange -- Image region to analysed
/*	Return:		0  -- Succeed
/***************************************************************************/
int CDirLine::BuildConnTree ( CImage& Img, RECT rRange )
{
    int  ImgType = Img.GetType() ;
    if ( ImgType!=DIB_1BIT )        return -1 ;

    int w = Img.GetWidth() ;
    int h = Img.GetHeight() ;
    int wb = Img.GetLineSizeInBytes() ;
	BOOL	bUpSideDown = Img.GetOrientation();
    if ( rRange.top<0 || rRange.top>=h || rRange.bottom<0 || rRange.bottom>=h || rRange.bottom<rRange.top ||
         rRange.left<0 || rRange.left>=w || rRange.right<0 || rRange.right>=w || rRange.right<rRange.left )
        return -3 ;

    int j, k, st, ed, span, valleys ;

    st = rRange.left ;
    ed = rRange.right ;
    span = rRange.bottom-rRange.top+1 ;

    m_Tree.Initialize(rRange);

    int *t = (int *)malloc(sizeof(int)*span+16) ;
    if ( t == (int *)NULL ) return -1;
    int *s = (int *)malloc(sizeof(int)*span+16) ;
    if ( s == (int *)NULL ) { free(t); return -1 ; }
    LineValley *v = (LineValley *)malloc(sizeof(LineValley)*span*2+16) ;
    if ( v == (LineValley *)NULL ) { free(s); free(t); return -1; }

    BYTE *p = Img.LockRawImg() ;
    if ( p==(BYTE *)NULL || wb<=0 )
	{ free(v); free(s); free(t); return -1; }
	
    for ( j=st ; j<=ed ; j++ )
    {
		valleys = GetColumnRunLength ( p, wb, h, j, rRange.top, rRange.bottom, v, bUpSideDown ) ;

        for ( k=0 ; k<valleys ; k++ )
		{
			v[k].x = j ;
			int vx = j;
			if( v[k].ye == rRange.bottom && v[k].ye < h-1 )
			{
				BYTE mask = 128>>(vx%8) ;
				while(v[k].ye < h-1 )
				{
					BYTE	DownLineByte = *( p+(h-1-v[k].ye-1)*wb+vx/8 );
					if ( (DownLineByte & mask) == 0 )
							break;
					v[k].ye++;
				}
			}
			if( v[k].ys == rRange.top && v[k].ys > 0 )
			{
				BYTE mask = 128>>(vx%8) ;
				while(v[k].ys > 0 )
				{
					BYTE	UpLineByte = *( p+(h-1-v[k].ys+1)*wb+vx/8 );
					if( (UpLineByte & mask) == 0 )	break;
					v[k].ys --;
				}
			}
		}
		m_Tree.AddNewCol ( v, valleys, j ) ;
	}
     
	free(v); 
	free(s); 
	free(t); 
	Img.UnlockRawImg();
    return 0 ;
}

/***************************************************************************
/*	Name:		AquireHorLineData
/*	Function:	Read a vertical image line data to build horizontal DSCC
*	Parameter:	p -- Pointer to image
/*				w -- Width of the whole image
/*				h -- Height of the whole image
/*				n -- X offset
/*				start -- Start line
/*				end -- End line
/*				buf -- Buffer for read data
/*				ImgType -- Image type
/*				Channel -- Channel for Color image
/*	Return:		0  -- Succeed
/***************************************************************************/
int CDirLine::AquireHorLineData ( BYTE *p, int w, int h, int n, int start, int end, int *buf, int ImgType, int Channel, BOOL bUpSideDown )
{
    int i ;
	if ( ImgType == DIB_1BIT )
	{
		BYTE mask = 128>>(n%8) ;
		if( bUpSideDown )
		{
			p += w*(h-1-start) + n/8 ;   // DIB: upside down
			for ( i=start ; i<=end ; i++, p-=w )
				if ( *p & mask )
					*buf++ = 0 ;
				else
					*buf++ = 255 ;
		}
		else
		{
			p += w*start + n/8;
			for ( i=start ; i<=end ; i++, p+=w )
				if ( *p & mask )
					*buf++ = 0 ;
				else
					*buf++ = 255 ;
		}
	}  
	else if ( ImgType == DIB_8BIT )
    {
		if( bUpSideDown )
		{
			p += w*(h-1-start) + n ;   // DIB: upside down
			for ( i=start ; i<=end ; i++, p-=w )
				*buf++ = *p ;
		}
		else
		{//Not upside down
			p += w*start + n;
			for( i=start; i<=end ; i++, p+=w )
				*buf++ = *p;
		}
    }
    else if ( ImgType == DIB_24BIT)
	{
		if( bUpSideDown )
		{
			p += w*(h-1-start) + 3*n;
			for ( i=start; i<=end ; i++, p-=w )
				*buf++ = *(p+Channel);
		}
		else
		{
			p += w*start + 3*n;
			for ( i=start; i<=end ; i++, p+=w )
				*buf++ = *(p+Channel);
		}
	}
	else
        return -1 ;
    return 0 ;
}

/***************************************************************************
/*	Name:		GetColumnRunLength
/*	Function:	Build column runlength
/*	Parameter:
/*	Return:		0  -- Succeed
/***************************************************************************/
int CDirLine::GetColumnRunLength ( BYTE *p, int w, int h, int column, int start, int end, LineValley *valley, BOOL bUpSideDown )
{
    int i, Valleys=0 ;
    BOOL bInValley ;

    BYTE mask = 128>>(column%8) ;
	if( bUpSideDown )
		p += w*(h-1-start) + column/8 ;   // DIB: upside down
	else
		p += w*start + column/8;

    for ( i=start, bInValley=FALSE ; i<=end ; i++ )
    {
        if ( *p & mask )
        {
            if ( !bInValley )
            {
                valley->ys = valley->yvs = i ;
                valley->EdgeGray = 255 ;
                valley->gray = 0 ;
                bInValley = TRUE ;
            }
        }
        else
        {
            if ( bInValley )
            {
                valley->ye = valley->yve = i-1 ;
                valley++ ;
                Valleys++ ;
                bInValley = FALSE ;
            }
        }
		if( i == end )	break;
		if( bUpSideDown )	p-=w;
		else				p+=w;
    }
    if ( bInValley )
    {
        valley->ye = valley->yve = i ;
        Valleys++ ;
    }
    return Valleys ;
}

/***************************************************************************
/*	Name:		GetConnComps
/*	Function:	Extract connected components
/*	Parameter:	Image     -- Image
/*				pConnComp -- Connected components
/*				nConnComp -- Number of connected components
/*	Return:		0  -- Succeed
/***************************************************************************/
int CDirLine::GetConnComps(CImage &Image, ConnComp *&pConnComp, int &nConnComp)
{
	m_Tree.m_IsHorConn = TRUE;	
	CRect rc;
	rc.left = 0;
	rc.right = Image.GetWidth()-1;
	rc.top = 0;
	rc.bottom = Image.GetHeight()-1;
	if ( BuildConnTree(Image, rc) != 0 )
		return -1 ;
	m_Tree.GetAllConnComps();
	pConnComp = m_Tree.m_pConnComp;
	nConnComp = m_Tree.m_nTotalConnComps;
	return 0 ;
}

/***************************************************************************
/*	Name:		GetConnComps
/*	Function:	Return number and pointer of connected components
/*	Parameter:	pConnComp -- Connected components
/*				nConnComp -- Number of connected components
/*	Return:		0  -- Succeed
/***************************************************************************/
int CDirLine::GetConnComps(ConnComp *&pConnComp, int &nConnComp)
{
	pConnComp = m_Tree.m_pConnComp;
	nConnComp = m_Tree.m_nTotalConnComps;
	return 0 ;
}

/***************************************************************************
/*	Name:		AddCompImage
/*	Function:	Add connected components into the output image
/*	Parameter:	pImg     -- Image
/*				wb       -- Image width in bytes
/*				h        -- Image height
/*				Offset   -- Offset of the connected component in the output image
/*				ConnComp -- Serial number of the processing connected componnets 
/*	Return:		0  -- Succeed
/***************************************************************************/
int	CDirLine::AddCompImage( BYTE *pImg, int wb, int h, CSize offset, int ConnComp )
{
	m_Tree.AddConnCompImage( pImg, wb, h, offset, ConnComp );
	return 0;
}
