////////////////////////////////////////////////////////////////////////////////////////////////
// File NAME:		WordSegment.h
// File Function:	Definition of class CWordSegment
//
//				Developed by: Zheng Yefeng
//			   First created: June, 2002
//			University of Maryland, College Park
//					All Right Reserved
///////////////////////////////////////////////////////////////////////////////////////////////////

#if !defined __WORD_SEGMENT_H__
#define __WORD_SEGMENT_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "define.h"
#include "DirLine.h"
#include "ImageObj.h"

class CWordSegment  
{
public:
	DOCSTRUM_PARAM	para;		//Parameter for word segmentation
	EST_PARA		est_para;	//Estimated parameters of character size etc.
	ZONE			*m_pComp;	//Connected components
	int				m_nComp;	//Number of connected components
	DIS				**knn_dis;	//K nearest connected compoents for each connected components
	ZONE			*m_pWord;	//Word
	int				m_nWord;	//Number of word
	CDirLine		DirLine;	//Used to extract connected components. Store all detailed information of connected components.
	int				m_nExtremeNoise;	//Noise blocks with extreme sizes
	BOOL			m_bChinese; //Chinese or English

public:
	int		FreeMem();
	int		SegmentWord( CImage &Img );
	CWordSegment();
	virtual ~CWordSegment();
	void	EstPara( );
	void	selectK(float a[], int b[], int n, int k);
	float	dis(ZONE a, ZONE b);
	float	ang(ZONE a, ZONE b);
	void	MakeWord( );
	void	TransitiveClosure(int E, int *x, int *y ) ;
	double  dis_manhantan(ZONE a, ZONE b);
	double  dis2(ZONE a, ZONE b);
	void	sort2(int n, float *ra, int *rb);
	void	visit(int k, NODE_DOCSTRUM *z, NODE_DOCSTRUM ***adj_p,	int *id_p, int **val_p, int *ch_index_p);
	int		LabelNoise( );
	int		SetWordSize( );
	void	SplitLargeWord( );
	int		MergeSmallWord( );
	int		ExtractConnComps( CImage &Img );
	int		PreserveExtremeNoiseBlock( );
	BOOL	SizeCompatible(ZONE a, ZONE b);
	int		OutputWordImage( CImage &ImgWord, int nWord );
	int		OutputImage( CImage &ImgOut, int nOutType );

	int		SetNoiseBlockConf( );

};

#endif // __WORD_SEGMENT_H__
