/////////////////////////////////////////////////////////////////////////
// File NAME:		ZoneSeg.h
// File Function:	Head file for the API for zone segmentation
//
//				Developed by:Yefeng Zheng
//					All Right Reserved
/////////////////////////////////////////////////////////////////////////

#ifndef	__ZONE_SEG_API_H__
#define __ZONE_SEG_API_H__

typedef struct {	/* Docstrum algorithm parameter data structure */
        int     knn;        /* number of nearest connected components to be clustered */
        int     lsize;      /* low size threshold for connected component */
        int     hsize;      /* high size threshold for connected component */
        int     hatol;      /* horizontal angle tolerance */
        int     vatol;      /* vertical angle tolerance */
        float   tcc;        /* threshold for clustering connected component into lines */
        float   par;        /* parallel threshold for clustering lines into zones */
        float   per;        /* perpendicular threshold for clustering lines into zones */
        float   scr;        /* super/sub script distance threshold */
        float   dis;        /* connected component size grouping threshold */
} DOCSTRUM_PARAM;

typedef struct {
  	int 	sr;
  	int 	nm;
  	float 	fr;
  	int 	ta;
  	int 	sw;
  	int 	Cw;
  	int 	Ch;
  	int 	Cr;
  	int 	Az;
  	int 	Al;
  	int 	Br;
} VORONOI_PARAM;

typedef struct { /* X-Y cut segmentation algorithm parameter data structure */
        int     Tnx;            /* X noise removal threshold */
        int     Tny;            /* Y noise removal threshold */
        int     Tcx;            /* X cut threshold */
        int     Tcy;            /* Y cut threshold */
} XYCUT_PARAM;

typedef struct {
	int x;		//X coordinate of the left top corner of the zone
	int y;		//Y coordinate of the left top corner of the zone
	int w;		//Width of the zone
	int h;		//Height of the zone
}ZONE;

class __declspec(dllexport) CZSZoneSeg 
{
private:
	ZONE *m_pZone;			//Segmented zones
	int	 m_nZone;			//Number of zones

public:
	CZSZoneSeg();
	~CZSZoneSeg();

public:
	int ZSSetImage( char *fnImg );				//Set image from file for segmentation
	int ZSSetImage( BYTE *Img, int w, int h );	//Set image from memory for segmentation
	int ZSDocstrum( DOCSTRUM_PARAM *para );		//Zone segmentation with the Docstrum algorithm
	int ZSVoronoi( VORONOI_PARAM *para );		//Zone segmentation with the Voronoi algorithm
	int ZSXycut( XYCUT_PARAM *para );			//Zone segmentation with the XY-Cut algorithm
	int ZSGetZoneSeg( ZONE *&pZone, int &nZone );//Get the zone segmentation results
	int ZSOutputZone( char *fnZone );			//Output zone segmentationresults to ZONE file
};
#endif //__ZONE_SEG_API_H__