/////////////////////////////////////////////////////////////////////////
// File NAME:		Tools.h
// File Function:	Commnon functions for other modules
//
//				Developed by:Zheng Yefeng
//					All Right Reserved
/////////////////////////////////////////////////////////////////////////

#ifndef __TOOLS_H__
#define __TOOLS_H__

#ifndef	PI
#define PI  3.1415926535
#endif

typedef struct TransParamStruc
{
	//Type==0  Linear transform, use dx, dy, Center, Angle to locate a form cell
	//Type==1 Bi-linear transform, use x0, y0, a,b,c,d,e,f to locate a form cell
	int		Type;	

	//Linear transform
	int		dx, dy;
	POINT	Center;
	double	Angle;	
	//Bi-linear transform
	double x0, y0; 
	double a,b,c,d,e,f;

} TRANSPARAM;
////////////////////////////////////////////////////////////////////////////////
//	File name manipulation functions
////////////////////////////////////////////////////////////////////////////////
//Change file extension
int			ChangeFileExt ( char *dest, char *src, char *ext ) ;
//Change file extension, another interface
int			ChangeFileExt ( CString	&dest, CString src, char *ext );
//Get file extension
CString		GetFileExtension ( CString FileName ) ;
//Get file name from a string
CString		GetFileName ( CString strFileName );
//Get file name (without extension) from a string
CString GetFileNameWithoutExt ( CString strFileName );
//Get file path from a string
CString		GetPathName ( CString strFileName );
//Get DLL name from a string
CString		GetDllName( CString strFileName );
//Get function name of a DLL file from a string
CString		GetDllFuncName ( CString strFileName );
//Test if a file exist or not
BOOL		FileExist( CString strFileName );
//Test if a directory exist or not
BOOL		DirExist( CString csDir );
///////////////////////////////////////////////////////////////////////////////
//	Sort data
//////////////////////////////////////////////////////////////////////////////
//Sort data for data type 'int'
void		SortData( int  Value[], int Num, BOOL bIncrease );
//Sort data for data type 'double'
void		SortData( double  Value[], int Num, BOOL bIncrease );
//Sort data for data type 'float'
void		SortData( float  Value[], int Num, BOOL bIncrease );
//Get middle vaule of an array for data type 'double'
double		GetMidValue ( double *Value, int nTotal ) ;
//Get middle vaule of an array for data type 'int'
int			GetMidValue ( int *Value, int nTotal ) ;

////////////////////////////////////////////////////////////////////////////////
//	Position relationship of points and lines
////////////////////////////////////////////////////////////////////////////////
//Shift a point
POINT		ShiftPoint ( POINT p, int dx, int dy );
//Transform a point using transform parameters
POINT		TransformPoint(POINT p, TRANSPARAM Param);
//Calculate the angle of a line formed by two points
double		GetAngle ( POINT Start, POINT End ) ;
//Get distance of two points
double		GetDistance ( POINT &pnt1, POINT &pnt2 ) ;
//Get distance of two points, interface for 'double' type
double		GetDistance ( double x1, double y1, double x2, double y2 ) ;
//Get distance of one point to a line formed by two points
double		GetDistance ( POINT &Pnt, POINT &StPnd, POINT &EdPnt);
//Get cross point of two lines
int			GetCrossPoint ( double x1, double y1, double x2, double y2, double _x1, double _y1, double _x2, double _y2,  POINT &CrossPnt);
//Get cross point of two lines
POINT		GetCrossPoint(POINT StPnt1, POINT EdPnt1, POINT StPnt2, POINT EdPnt2);
//Get cross point one point to a line formed by two points
POINT		GetCrossPoint(POINT &pnt, POINT &StPnt, POINT &EdPnt);

///////////////////////////////////////////////////////////////////////////////////////
//	Matrix operation
////////////////////////////////////////////////////////////////////////////////////////
//Inverse a matrix
int		Inv(double a[], int n);
//Transpose a matrix
int		TransMatrix( double* A, int M, int N, double* B );
//Calculate feature mean
int		Mean( double **Feature, int Num, int Dimension, double *MeanFeature );
//Calculate feature mean
int		Mean( double *Feature, int Num, double &MeanFeature );
//Calculate variance matrix of feature
int		Var( double **Feature, int Num, int Dimension, double *Var, double *MeanFeature );
//Calculate variance matrix of feature
int		Var( double *Feature, int Num, double &Var, double &MeanFeature );
//Calculate determinate of a matrix
double	det(double a[], int n);
//Matrix multiplication
double	MatrixMultiply( double *a, double *b, double *c, int m, int n, int k);

///////////////////////////////////////////////////////////////////////////////
//	Other functions
////////////////////////////////////////////////////////////////////////////////
//Get four positions from a string
int			ParsePosition ( CString s, int &left, int &top, int &right, int &bottom, int type );
//Get a point position from a string 
int			ParseCenter( CString s, int &x, int &y);
//Inialize lut[] to calucate the number of black pixels in a byte
void		BlackPixelsInByte();
//Search peaks
int			SearchPeak( double *Hist, int nHist, int *&LeftPeak, int *&RightPeak, int &nPeak);
//Search valleys
int			SearchValley( double *Hist, int nHist, int *&LeftValley, int *&RightValley, int &nValley);
//Calculate peak width, do not consider background noise
int			GetPeakWidth2( double *Hist, int nHist, int nPeakPos );
//Calculate peak width
int			GetPeakWidth( double *Hist, int nHist, int nPeakPos );
//Calculate peak height
int			GetPeakHeight( double *Hist, int nHist, int nPeakPos );



#endif // __TOOLS_H__
